# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from .. import services

import types

__config__ = pulumi.Config('render')


class _ExportableConfig(types.ModuleType):
    @property
    def api_key(self) -> Optional[str]:
        """
        The Render API key
        """
        return __config__.get('apiKey')

    @property
    def clear_cache_on_service_update_deployments(self) -> Optional[str]:
        """
        When a service is updated, a deployment is automatically triggered. This variable controls whether or not the service cache should be cleared upon deployment.
        """
        return __config__.get('clearCacheOnServiceUpdateDeployments')




# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from ._enums import *

__all__ = [
    'CidrBlockAndDescriptionArgs',
    'ReadReplicaInputArgs',
]

@pulumi.input_type
class CidrBlockAndDescriptionArgs:
    def __init__(__self__, *,
                 cidr_block: pulumi.Input[str],
                 description: pulumi.Input[str]):
        """
        :param pulumi.Input[str] description: User-provided description of the CIDR block
        """
        pulumi.set(__self__, "cidr_block", cidr_block)
        pulumi.set(__self__, "description", description)

    @property
    @pulumi.getter(name="cidrBlock")
    def cidr_block(self) -> pulumi.Input[str]:
        return pulumi.get(self, "cidr_block")

    @cidr_block.setter
    def cidr_block(self, value: pulumi.Input[str]):
        pulumi.set(self, "cidr_block", value)

    @property
    @pulumi.getter
    def description(self) -> pulumi.Input[str]:
        """
        User-provided description of the CIDR block
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: pulumi.Input[str]):
        pulumi.set(self, "description", value)


@pulumi.input_type
class ReadReplicaInputArgs:
    def __init__(__self__, *,
                 name: pulumi.Input[str]):
        """
        :param pulumi.Input[str] name: The display name of the replica instance.
        """
        pulumi.set(__self__, "name", name)

    @property
    @pulumi.getter
    def name(self) -> pulumi.Input[str]:
        """
        The display name of the replica instance.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: pulumi.Input[str]):
        pulumi.set(self, "name", value)





# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from . import _utilities
import typing
# Export this package's modules as members:
from .provider import *

# Make subpackages available:
if typing.TYPE_CHECKING:
    import pulumi_render.blueprints as __blueprints
    blueprints = __blueprints
    import pulumi_render.config as __config
    config = __config
    import pulumi_render.cron_jobs as __cron_jobs
    cron_jobs = __cron_jobs
    import pulumi_render.disks as __disks
    disks = __disks
    import pulumi_render.env_groups as __env_groups
    env_groups = __env_groups
    import pulumi_render.environments as __environments
    environments = __environments
    import pulumi_render.metrics as __metrics
    metrics = __metrics
    import pulumi_render.notification_settings as __notification_settings
    notification_settings = __notification_settings
    import pulumi_render.owners as __owners
    owners = __owners
    import pulumi_render.postgres as __postgres
    postgres = __postgres
    import pulumi_render.projects as __projects
    projects = __projects
    import pulumi_render.redis as __redis
    redis = __redis
    import pulumi_render.registrycredentials as __registrycredentials
    registrycredentials = __registrycredentials
    import pulumi_render.services as __services
    services = __services
else:
    blueprints = _utilities.lazy_import('pulumi_render.blueprints')
    config = _utilities.lazy_import('pulumi_render.config')
    cron_jobs = _utilities.lazy_import('pulumi_render.cron_jobs')
    disks = _utilities.lazy_import('pulumi_render.disks')
    env_groups = _utilities.lazy_import('pulumi_render.env_groups')
    environments = _utilities.lazy_import('pulumi_render.environments')
    metrics = _utilities.lazy_import('pulumi_render.metrics')
    notification_settings = _utilities.lazy_import('pulumi_render.notification_settings')
    owners = _utilities.lazy_import('pulumi_render.owners')
    postgres = _utilities.lazy_import('pulumi_render.postgres')
    projects = _utilities.lazy_import('pulumi_render.projects')
    redis = _utilities.lazy_import('pulumi_render.redis')
    registrycredentials = _utilities.lazy_import('pulumi_render.registrycredentials')
    services = _utilities.lazy_import('pulumi_render.services')

_utilities.register(
    resource_modules="""
[
 {
  "pkg": "render",
  "mod": "cron-jobs",
  "fqn": "pulumi_render.cron_jobs",
  "classes": {
   "render:cron-jobs:RunCronJob": "RunCronJob"
  }
 },
 {
  "pkg": "render",
  "mod": "disks",
  "fqn": "pulumi_render.disks",
  "classes": {
   "render:disks:Disk": "Disk",
   "render:disks:RestoreSnapshot": "RestoreSnapshot"
  }
 },
 {
  "pkg": "render",
  "mod": "env-groups",
  "fqn": "pulumi_render.env_groups",
  "classes": {
   "render:env-groups:EnvGroup": "EnvGroup",
   "render:env-groups:LinkServiceToEnvGroup": "LinkServiceToEnvGroup"
  }
 },
 {
  "pkg": "render",
  "mod": "environments",
  "fqn": "pulumi_render.environments",
  "classes": {
   "render:environments:Environment": "Environment",
   "render:environments:ResourcesToEnvironment": "ResourcesToEnvironment"
  }
 },
 {
  "pkg": "render",
  "mod": "postgres",
  "fqn": "pulumi_render.postgres",
  "classes": {
   "render:postgres:FailoverPostgres": "FailoverPostgres",
   "render:postgres:Postgres": "Postgres",
   "render:postgres:PostgresBackup": "PostgresBackup",
   "render:postgres:RecoverPostgres": "RecoverPostgres",
   "render:postgres:RestartPostgres": "RestartPostgres",
   "render:postgres:ResumePostgres": "ResumePostgres",
   "render:postgres:SuspendPostgres": "SuspendPostgres"
  }
 },
 {
  "pkg": "render",
  "mod": "projects",
  "fqn": "pulumi_render.projects",
  "classes": {
   "render:projects:Project": "Project"
  }
 },
 {
  "pkg": "render",
  "mod": "redis",
  "fqn": "pulumi_render.redis",
  "classes": {
   "render:redis:Redis": "Redis"
  }
 },
 {
  "pkg": "render",
  "mod": "registrycredentials",
  "fqn": "pulumi_render.registrycredentials",
  "classes": {
   "render:registrycredentials:RegistryCredential": "RegistryCredential"
  }
 },
 {
  "pkg": "render",
  "mod": "services",
  "fqn": "pulumi_render.services",
  "classes": {
   "render:services:AutoscaleService": "AutoscaleService",
   "render:services:BackgroundWorker": "BackgroundWorker",
   "render:services:CancelDeploy": "CancelDeploy",
   "render:services:CancelJob": "CancelJob",
   "render:services:CronJob": "CronJob",
   "render:services:CustomDomain": "CustomDomain",
   "render:services:Deploy": "Deploy",
   "render:services:EnvVarsForService": "EnvVarsForService",
   "render:services:Header": "Header",
   "render:services:Job": "Job",
   "render:services:PreviewService": "PreviewService",
   "render:services:PrivateService": "PrivateService",
   "render:services:RefreshCustomDomain": "RefreshCustomDomain",
   "render:services:RestartService": "RestartService",
   "render:services:RollbackDeploy": "RollbackDeploy",
   "render:services:Route": "Route",
   "render:services:ScaleService": "ScaleService",
   "render:services:SecretFilesForService": "SecretFilesForService",
   "render:services:StaticSite": "StaticSite",
   "render:services:SuspendService": "SuspendService",
   "render:services:WebService": "WebService"
  }
 }
]
""",
    resource_packages="""
[
 {
  "pkg": "render",
  "token": "pulumi:providers:render",
  "fqn": "pulumi_render",
  "class": "Provider"
 }
]
"""
)



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from ._enums import *

__all__ = ['RegistryCredentialArgs', 'RegistryCredential']

@pulumi.input_type
class RegistryCredentialArgs:
    def __init__(__self__, *,
                 auth_token: pulumi.Input[str],
                 owner_id: pulumi.Input[str],
                 registry: pulumi.Input['Registry'],
                 username: pulumi.Input[str],
                 name: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a RegistryCredential resource.
        :param pulumi.Input['Registry'] registry: The registry to use this credential with
        """
        pulumi.set(__self__, "auth_token", auth_token)
        pulumi.set(__self__, "owner_id", owner_id)
        pulumi.set(__self__, "registry", registry)
        pulumi.set(__self__, "username", username)
        if name is not None:
            pulumi.set(__self__, "name", name)

    @property
    @pulumi.getter(name="authToken")
    def auth_token(self) -> pulumi.Input[str]:
        return pulumi.get(self, "auth_token")

    @auth_token.setter
    def auth_token(self, value: pulumi.Input[str]):
        pulumi.set(self, "auth_token", value)

    @property
    @pulumi.getter(name="ownerId")
    def owner_id(self) -> pulumi.Input[str]:
        return pulumi.get(self, "owner_id")

    @owner_id.setter
    def owner_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "owner_id", value)

    @property
    @pulumi.getter
    def registry(self) -> pulumi.Input['Registry']:
        """
        The registry to use this credential with
        """
        return pulumi.get(self, "registry")

    @registry.setter
    def registry(self, value: pulumi.Input['Registry']):
        pulumi.set(self, "registry", value)

    @property
    @pulumi.getter
    def username(self) -> pulumi.Input[str]:
        return pulumi.get(self, "username")

    @username.setter
    def username(self, value: pulumi.Input[str]):
        pulumi.set(self, "username", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)


class RegistryCredential(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 auth_token: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 owner_id: Optional[pulumi.Input[str]] = None,
                 registry: Optional[pulumi.Input['Registry']] = None,
                 username: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Create a RegistryCredential resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input['Registry'] registry: The registry to use this credential with
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: RegistryCredentialArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Create a RegistryCredential resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param RegistryCredentialArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(RegistryCredentialArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 auth_token: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 owner_id: Optional[pulumi.Input[str]] = None,
                 registry: Optional[pulumi.Input['Registry']] = None,
                 username: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = RegistryCredentialArgs.__new__(RegistryCredentialArgs)

            if auth_token is None and not opts.urn:
                raise TypeError("Missing required property 'auth_token'")
            __props__.__dict__["auth_token"] = auth_token
            __props__.__dict__["name"] = name
            if owner_id is None and not opts.urn:
                raise TypeError("Missing required property 'owner_id'")
            __props__.__dict__["owner_id"] = owner_id
            if registry is None and not opts.urn:
                raise TypeError("Missing required property 'registry'")
            __props__.__dict__["registry"] = registry
            if username is None and not opts.urn:
                raise TypeError("Missing required property 'username'")
            __props__.__dict__["username"] = username
        super(RegistryCredential, __self__).__init__(
            'render:registrycredentials:RegistryCredential',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None) -> 'RegistryCredential':
        """
        Get an existing RegistryCredential resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = RegistryCredentialArgs.__new__(RegistryCredentialArgs)

        __props__.__dict__["auth_token"] = None
        __props__.__dict__["name"] = None
        __props__.__dict__["owner_id"] = None
        __props__.__dict__["registry"] = None
        __props__.__dict__["username"] = None
        return RegistryCredential(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="authToken")
    def auth_token(self) -> pulumi.Output[str]:
        return pulumi.get(self, "auth_token")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        Descriptive name for this credential
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="ownerId")
    def owner_id(self) -> pulumi.Output[str]:
        return pulumi.get(self, "owner_id")

    @property
    @pulumi.getter
    def registry(self) -> pulumi.Output['Registry']:
        """
        The registry to use this credential with
        """
        return pulumi.get(self, "registry")

    @property
    @pulumi.getter
    def username(self) -> pulumi.Output[str]:
        """
        The username associated with the credential
        """
        return pulumi.get(self, "username")




# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload, Awaitable
from .. import _utilities
from ._enums import *

__all__ = [
    'Environment',
    'AwaitableEnvironment',
    'get_environment',
    'get_environment_output',
]

@pulumi.output_type
class Environment:
    def __init__(__self__, databases_ids=None, env_group_ids=None, id=None, name=None, project_id=None, protected_status=None, redis_ids=None, service_ids=None):
        if databases_ids and not isinstance(databases_ids, list):
            raise TypeError("Expected argument 'databases_ids' to be a list")
        pulumi.set(__self__, "databases_ids", databases_ids)
        if env_group_ids and not isinstance(env_group_ids, list):
            raise TypeError("Expected argument 'env_group_ids' to be a list")
        pulumi.set(__self__, "env_group_ids", env_group_ids)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if project_id and not isinstance(project_id, str):
            raise TypeError("Expected argument 'project_id' to be a str")
        pulumi.set(__self__, "project_id", project_id)
        if protected_status and not isinstance(protected_status, str):
            raise TypeError("Expected argument 'protected_status' to be a str")
        pulumi.set(__self__, "protected_status", protected_status)
        if redis_ids and not isinstance(redis_ids, list):
            raise TypeError("Expected argument 'redis_ids' to be a list")
        pulumi.set(__self__, "redis_ids", redis_ids)
        if service_ids and not isinstance(service_ids, list):
            raise TypeError("Expected argument 'service_ids' to be a list")
        pulumi.set(__self__, "service_ids", service_ids)

    @property
    @pulumi.getter(name="databasesIds")
    def databases_ids(self) -> Sequence[str]:
        return pulumi.get(self, "databases_ids")

    @property
    @pulumi.getter(name="envGroupIds")
    def env_group_ids(self) -> Sequence[str]:
        return pulumi.get(self, "env_group_ids")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="projectId")
    def project_id(self) -> str:
        return pulumi.get(self, "project_id")

    @property
    @pulumi.getter(name="protectedStatus")
    def protected_status(self) -> 'EnvironmentProtectedStatus':
        """
        Indicates whether an environment is `unprotected` or `protected`. Only admin users can perform destructive actions in `protected` environments.
        """
        return pulumi.get(self, "protected_status")

    @property
    @pulumi.getter(name="redisIds")
    def redis_ids(self) -> Sequence[str]:
        return pulumi.get(self, "redis_ids")

    @property
    @pulumi.getter(name="serviceIds")
    def service_ids(self) -> Sequence[str]:
        return pulumi.get(self, "service_ids")


class AwaitableEnvironment(Environment):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return Environment(
            databases_ids=self.databases_ids,
            env_group_ids=self.env_group_ids,
            id=self.id,
            name=self.name,
            project_id=self.project_id,
            protected_status=self.protected_status,
            redis_ids=self.redis_ids,
            service_ids=self.service_ids)


def get_environment(environment_id: Optional[str] = None,
                    opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableEnvironment:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['environmentId'] = environment_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:environments:getEnvironment', __args__, opts=opts, typ=Environment).value

    return AwaitableEnvironment(
        databases_ids=pulumi.get(__ret__, 'databases_ids'),
        env_group_ids=pulumi.get(__ret__, 'env_group_ids'),
        id=pulumi.get(__ret__, 'id'),
        name=pulumi.get(__ret__, 'name'),
        project_id=pulumi.get(__ret__, 'project_id'),
        protected_status=pulumi.get(__ret__, 'protected_status'),
        redis_ids=pulumi.get(__ret__, 'redis_ids'),
        service_ids=pulumi.get(__ret__, 'service_ids'))


@_utilities.lift_output_func(get_environment)
def get_environment_output(environment_id: Optional[pulumi.Input[str]] = None,
                           opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[Environment]:
    """
    Use this data source to access information about an existing resource.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from enum import Enum

__all__ = [
    'Status',
]


class Status(str, Enum):
    PENDING = "pending"
    SUCCESSFUL = "successful"
    UNSUCCESSFUL = "unsuccessful"
    CANCELED = "canceled"



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'CidrBlockAndDescription',
    'ListPostgresBackupItemProperties',
    'ListPostgresItemProperties',
    'Owner',
    'Postgres',
    'ReadReplica',
    'RedisDetailpropertiesmaintenance',
]

@pulumi.output_type
class CidrBlockAndDescription(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "cidrBlock":
            suggest = "cidr_block"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in CidrBlockAndDescription. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        CidrBlockAndDescription.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        CidrBlockAndDescription.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 cidr_block: str,
                 description: str):
        """
        :param str description: User-provided description of the CIDR block
        """
        pulumi.set(__self__, "cidr_block", cidr_block)
        pulumi.set(__self__, "description", description)

    @property
    @pulumi.getter(name="cidrBlock")
    def cidr_block(self) -> str:
        return pulumi.get(self, "cidr_block")

    @property
    @pulumi.getter
    def description(self) -> str:
        """
        User-provided description of the CIDR block
        """
        return pulumi.get(self, "description")


@pulumi.output_type
class ListPostgresBackupItemProperties(dict):
    def __init__(__self__, *,
                 created_at: str,
                 id: str,
                 url: Optional[str] = None):
        """
        :param str url: URL to download the Postgres backup
        """
        pulumi.set(__self__, "created_at", created_at)
        pulumi.set(__self__, "id", id)
        if url is not None:
            pulumi.set(__self__, "url", url)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> str:
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def url(self) -> Optional[str]:
        """
        URL to download the Postgres backup
        """
        return pulumi.get(self, "url")


@pulumi.output_type
class ListPostgresItemProperties(dict):
    def __init__(__self__, *,
                 cursor: Optional[str] = None,
                 postgres: Optional['outputs.Postgres'] = None):
        if cursor is not None:
            pulumi.set(__self__, "cursor", cursor)
        if postgres is not None:
            pulumi.set(__self__, "postgres", postgres)

    @property
    @pulumi.getter
    def cursor(self) -> Optional[str]:
        return pulumi.get(self, "cursor")

    @property
    @pulumi.getter
    def postgres(self) -> Optional['outputs.Postgres']:
        return pulumi.get(self, "postgres")


@pulumi.output_type
class Owner(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "twoFactorAuthEnabled":
            suggest = "two_factor_auth_enabled"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in Owner. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        Owner.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        Owner.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 email: str,
                 id: str,
                 name: str,
                 type: 'OwnerType',
                 two_factor_auth_enabled: Optional[bool] = None):
        """
        :param bool two_factor_auth_enabled: Whether two-factor authentication is enabled for the owner. Only present for user owners.
        """
        pulumi.set(__self__, "email", email)
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "type", type)
        if two_factor_auth_enabled is not None:
            pulumi.set(__self__, "two_factor_auth_enabled", two_factor_auth_enabled)

    @property
    @pulumi.getter
    def email(self) -> str:
        return pulumi.get(self, "email")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def type(self) -> 'OwnerType':
        return pulumi.get(self, "type")

    @property
    @pulumi.getter(name="twoFactorAuthEnabled")
    def two_factor_auth_enabled(self) -> Optional[bool]:
        """
        Whether two-factor authentication is enabled for the owner. Only present for user owners.
        """
        return pulumi.get(self, "two_factor_auth_enabled")


@pulumi.output_type
class Postgres(dict):
    def __init__(__self__, *,
                 created_at: str,
                 dashboard_url: str,
                 database_name: str,
                 database_user: str,
                 high_availability_enabled: bool,
                 id: str,
                 ip_allow_list: Sequence['outputs.CidrBlockAndDescription'],
                 name: str,
                 owner: 'outputs.Owner',
                 plan: 'PostgresPlan',
                 read_replicas: Sequence['outputs.ReadReplica'],
                 region: Optional['PostgresRegion'] = None,
                 role: 'PostgresRole',
                 status: 'PostgresStatus',
                 suspended: 'PostgresSuspended',
                 suspenders: Sequence['PostgresSuspendersItem'],
                 updated_at: str,
                 version: 'PostgresVersion',
                 disk_size_gb: Optional[int] = None,
                 environment_id: Optional[str] = None,
                 expires_at: Optional[str] = None,
                 primary_postgres_id: Optional[str] = None):
        """
        :param str dashboard_url: The URL to view the PostgreSQL instance in the Render Dashboard
        :param 'PostgresRegion' region: Defaults to "oregon"
        :param 'PostgresVersion' version: The PostgreSQL version
        :param str expires_at: The time at which the database will be expire. Applies to free tier databases only.
        """
        pulumi.set(__self__, "created_at", created_at)
        pulumi.set(__self__, "dashboard_url", dashboard_url)
        pulumi.set(__self__, "database_name", database_name)
        pulumi.set(__self__, "database_user", database_user)
        pulumi.set(__self__, "high_availability_enabled", high_availability_enabled)
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "ip_allow_list", ip_allow_list)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "owner", owner)
        pulumi.set(__self__, "plan", plan)
        pulumi.set(__self__, "read_replicas", read_replicas)
        if region is None:
            region = 'oregon'
        pulumi.set(__self__, "region", region)
        pulumi.set(__self__, "role", role)
        pulumi.set(__self__, "status", status)
        pulumi.set(__self__, "suspended", suspended)
        pulumi.set(__self__, "suspenders", suspenders)
        pulumi.set(__self__, "updated_at", updated_at)
        pulumi.set(__self__, "version", version)
        if disk_size_gb is not None:
            pulumi.set(__self__, "disk_size_gb", disk_size_gb)
        if environment_id is not None:
            pulumi.set(__self__, "environment_id", environment_id)
        if expires_at is not None:
            pulumi.set(__self__, "expires_at", expires_at)
        if primary_postgres_id is not None:
            pulumi.set(__self__, "primary_postgres_id", primary_postgres_id)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> str:
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter(name="dashboardUrl")
    def dashboard_url(self) -> str:
        """
        The URL to view the PostgreSQL instance in the Render Dashboard
        """
        return pulumi.get(self, "dashboard_url")

    @property
    @pulumi.getter(name="databaseName")
    def database_name(self) -> str:
        return pulumi.get(self, "database_name")

    @property
    @pulumi.getter(name="databaseUser")
    def database_user(self) -> str:
        return pulumi.get(self, "database_user")

    @property
    @pulumi.getter(name="highAvailabilityEnabled")
    def high_availability_enabled(self) -> bool:
        return pulumi.get(self, "high_availability_enabled")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="ipAllowList")
    def ip_allow_list(self) -> Sequence['outputs.CidrBlockAndDescription']:
        return pulumi.get(self, "ip_allow_list")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def owner(self) -> 'outputs.Owner':
        return pulumi.get(self, "owner")

    @property
    @pulumi.getter
    def plan(self) -> 'PostgresPlan':
        return pulumi.get(self, "plan")

    @property
    @pulumi.getter(name="readReplicas")
    def read_replicas(self) -> Sequence['outputs.ReadReplica']:
        return pulumi.get(self, "read_replicas")

    @property
    @pulumi.getter
    def region(self) -> 'PostgresRegion':
        """
        Defaults to "oregon"
        """
        return pulumi.get(self, "region")

    @property
    @pulumi.getter
    def role(self) -> 'PostgresRole':
        return pulumi.get(self, "role")

    @property
    @pulumi.getter
    def status(self) -> 'PostgresStatus':
        return pulumi.get(self, "status")

    @property
    @pulumi.getter
    def suspended(self) -> 'PostgresSuspended':
        return pulumi.get(self, "suspended")

    @property
    @pulumi.getter
    def suspenders(self) -> Sequence['PostgresSuspendersItem']:
        return pulumi.get(self, "suspenders")

    @property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> str:
        return pulumi.get(self, "updated_at")

    @property
    @pulumi.getter
    def version(self) -> 'PostgresVersion':
        """
        The PostgreSQL version
        """
        return pulumi.get(self, "version")

    @property
    @pulumi.getter(name="diskSizeGB")
    def disk_size_gb(self) -> Optional[int]:
        return pulumi.get(self, "disk_size_gb")

    @property
    @pulumi.getter(name="environmentId")
    def environment_id(self) -> Optional[str]:
        return pulumi.get(self, "environment_id")

    @property
    @pulumi.getter(name="expiresAt")
    def expires_at(self) -> Optional[str]:
        """
        The time at which the database will be expire. Applies to free tier databases only.
        """
        return pulumi.get(self, "expires_at")

    @property
    @pulumi.getter(name="primaryPostgresID")
    def primary_postgres_id(self) -> Optional[str]:
        return pulumi.get(self, "primary_postgres_id")


@pulumi.output_type
class ReadReplica(dict):
    def __init__(__self__, *,
                 id: str,
                 name: str):
        """
        :param str id: The replica instance identifier.
        :param str name: The display name of the replica instance.
        """
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "name", name)

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The replica instance identifier.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        """
        The display name of the replica instance.
        """
        return pulumi.get(self, "name")


@pulumi.output_type
class RedisDetailpropertiesmaintenance(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "scheduledAt":
            suggest = "scheduled_at"
        elif key == "pendingMaintenanceBy":
            suggest = "pending_maintenance_by"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in RedisDetailpropertiesmaintenance. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        RedisDetailpropertiesmaintenance.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        RedisDetailpropertiesmaintenance.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 id: str,
                 scheduled_at: str,
                 state: 'RedisDetailpropertiesmaintenanceState',
                 type: str,
                 pending_maintenance_by: Optional[str] = None):
        """
        :param str pending_maintenance_by: If present, the maintenance run cannot be scheduled for later than this date-time.
        """
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "scheduled_at", scheduled_at)
        pulumi.set(__self__, "state", state)
        pulumi.set(__self__, "type", type)
        if pending_maintenance_by is not None:
            pulumi.set(__self__, "pending_maintenance_by", pending_maintenance_by)

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="scheduledAt")
    def scheduled_at(self) -> str:
        return pulumi.get(self, "scheduled_at")

    @property
    @pulumi.getter
    def state(self) -> 'RedisDetailpropertiesmaintenanceState':
        return pulumi.get(self, "state")

    @property
    @pulumi.getter
    def type(self) -> str:
        return pulumi.get(self, "type")

    @property
    @pulumi.getter(name="pendingMaintenanceBy")
    def pending_maintenance_by(self) -> Optional[str]:
        """
        If present, the maintenance run cannot be scheduled for later than this date-time.
        """
        return pulumi.get(self, "pending_maintenance_by")





# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'NotificationOverrideWithCursor',
    'NotificationOverrideWithCursorOverrideProperties',
]

@pulumi.output_type
class NotificationOverrideWithCursor(dict):
    def __init__(__self__, *,
                 cursor: str,
                 override: 'outputs.NotificationOverrideWithCursorOverrideProperties'):
        pulumi.set(__self__, "cursor", cursor)
        pulumi.set(__self__, "override", override)

    @property
    @pulumi.getter
    def cursor(self) -> str:
        return pulumi.get(self, "cursor")

    @property
    @pulumi.getter
    def override(self) -> 'outputs.NotificationOverrideWithCursorOverrideProperties':
        return pulumi.get(self, "override")


@pulumi.output_type
class NotificationOverrideWithCursorOverrideProperties(dict):
    def __init__(__self__, *,
                 notifications_to_send: 'NotificationOverrideWithCursorOverridePropertiesNotificationsToSend',
                 preview_notifications_enabled: 'NotificationOverrideWithCursorOverridePropertiesPreviewNotificationsEnabled',
                 service_id: str):
        pulumi.set(__self__, "notifications_to_send", notifications_to_send)
        pulumi.set(__self__, "preview_notifications_enabled", preview_notifications_enabled)
        pulumi.set(__self__, "service_id", service_id)

    @property
    @pulumi.getter(name="notificationsToSend")
    def notifications_to_send(self) -> 'NotificationOverrideWithCursorOverridePropertiesNotificationsToSend':
        return pulumi.get(self, "notifications_to_send")

    @property
    @pulumi.getter(name="previewNotificationsEnabled")
    def preview_notifications_enabled(self) -> 'NotificationOverrideWithCursorOverridePropertiesPreviewNotificationsEnabled':
        return pulumi.get(self, "preview_notifications_enabled")

    @property
    @pulumi.getter(name="serviceId")
    def service_id(self) -> str:
        return pulumi.get(self, "service_id")





# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from ._enums import *

__all__ = [
    'ProjectCreateEnvironmentInputArgs',
]

@pulumi.input_type
class ProjectCreateEnvironmentInputArgs:
    def __init__(__self__, *,
                 name: pulumi.Input[str],
                 protected_status: Optional[pulumi.Input['ProjectCreateEnvironmentInputProtectedStatus']] = None):
        """
        :param pulumi.Input['ProjectCreateEnvironmentInputProtectedStatus'] protected_status: Indicates whether an environment is `unprotected` or `protected`. Only admin users can perform destructive actions in `protected` environments.
        """
        pulumi.set(__self__, "name", name)
        if protected_status is not None:
            pulumi.set(__self__, "protected_status", protected_status)

    @property
    @pulumi.getter
    def name(self) -> pulumi.Input[str]:
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: pulumi.Input[str]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="protectedStatus")
    def protected_status(self) -> Optional[pulumi.Input['ProjectCreateEnvironmentInputProtectedStatus']]:
        """
        Indicates whether an environment is `unprotected` or `protected`. Only admin users can perform destructive actions in `protected` environments.
        """
        return pulumi.get(self, "protected_status")

    @protected_status.setter
    def protected_status(self, value: Optional[pulumi.Input['ProjectCreateEnvironmentInputProtectedStatus']]):
        pulumi.set(self, "protected_status", value)





# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload, Awaitable
from .. import _utilities

__all__ = [
    'RedisConnectionInfo',
    'AwaitableRedisConnectionInfo',
    'get_redis_connection_info',
    'get_redis_connection_info_output',
]

@pulumi.output_type
class RedisConnectionInfo:
    """
    A Redis instance
    """
    def __init__(__self__, external_connection_string=None, internal_connection_string=None, redis_cli_command=None):
        if external_connection_string and not isinstance(external_connection_string, str):
            raise TypeError("Expected argument 'external_connection_string' to be a str")
        pulumi.set(__self__, "external_connection_string", external_connection_string)
        if internal_connection_string and not isinstance(internal_connection_string, str):
            raise TypeError("Expected argument 'internal_connection_string' to be a str")
        pulumi.set(__self__, "internal_connection_string", internal_connection_string)
        if redis_cli_command and not isinstance(redis_cli_command, str):
            raise TypeError("Expected argument 'redis_cli_command' to be a str")
        pulumi.set(__self__, "redis_cli_command", redis_cli_command)

    @property
    @pulumi.getter(name="externalConnectionString")
    def external_connection_string(self) -> str:
        """
        The connection string to use from outside Render
        """
        return pulumi.get(self, "external_connection_string")

    @property
    @pulumi.getter(name="internalConnectionString")
    def internal_connection_string(self) -> str:
        """
        The connection string to use from within Render
        """
        return pulumi.get(self, "internal_connection_string")

    @property
    @pulumi.getter(name="redisCLICommand")
    def redis_cli_command(self) -> str:
        """
        The Redis CLI command to connect to the Redis instance
        """
        return pulumi.get(self, "redis_cli_command")


class AwaitableRedisConnectionInfo(RedisConnectionInfo):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return RedisConnectionInfo(
            external_connection_string=self.external_connection_string,
            internal_connection_string=self.internal_connection_string,
            redis_cli_command=self.redis_cli_command)


def get_redis_connection_info(redis_id: Optional[str] = None,
                              opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableRedisConnectionInfo:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['redisId'] = redis_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:redis:getRedisConnectionInfo', __args__, opts=opts, typ=RedisConnectionInfo).value

    return AwaitableRedisConnectionInfo(
        external_connection_string=pulumi.get(__ret__, 'external_connection_string'),
        internal_connection_string=pulumi.get(__ret__, 'internal_connection_string'),
        redis_cli_command=pulumi.get(__ret__, 'redis_cli_command'))


@_utilities.lift_output_func(get_redis_connection_info)
def get_redis_connection_info_output(redis_id: Optional[pulumi.Input[str]] = None,
                                     opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[RedisConnectionInfo]:
    """
    Use this data source to access information about an existing resource.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *
from ._inputs import *

__all__ = ['RedisArgs', 'Redis']

@pulumi.input_type
class RedisArgs:
    def __init__(__self__, *,
                 owner_id: pulumi.Input[str],
                 plan: pulumi.Input['Plan'],
                 environment_id: Optional[pulumi.Input[str]] = None,
                 ip_allow_list: Optional[pulumi.Input[Sequence[pulumi.Input['CidrBlockAndDescriptionArgs']]]] = None,
                 maxmemory_policy: Optional[pulumi.Input['MaxmemoryPolicy']] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 region: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a Redis resource.
        :param pulumi.Input[str] owner_id: The ID of the owner of the Redis instance
        :param pulumi.Input['MaxmemoryPolicy'] maxmemory_policy: The eviction policy for the Redis instance
        :param pulumi.Input[str] name: The name of the Redis instance
        :param pulumi.Input[str] region: The region where the Redis instance is located
        """
        pulumi.set(__self__, "owner_id", owner_id)
        pulumi.set(__self__, "plan", plan)
        if environment_id is not None:
            pulumi.set(__self__, "environment_id", environment_id)
        if ip_allow_list is not None:
            pulumi.set(__self__, "ip_allow_list", ip_allow_list)
        if maxmemory_policy is not None:
            pulumi.set(__self__, "maxmemory_policy", maxmemory_policy)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if region is not None:
            pulumi.set(__self__, "region", region)

    @property
    @pulumi.getter(name="ownerId")
    def owner_id(self) -> pulumi.Input[str]:
        """
        The ID of the owner of the Redis instance
        """
        return pulumi.get(self, "owner_id")

    @owner_id.setter
    def owner_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "owner_id", value)

    @property
    @pulumi.getter
    def plan(self) -> pulumi.Input['Plan']:
        return pulumi.get(self, "plan")

    @plan.setter
    def plan(self, value: pulumi.Input['Plan']):
        pulumi.set(self, "plan", value)

    @property
    @pulumi.getter(name="environmentId")
    def environment_id(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "environment_id")

    @environment_id.setter
    def environment_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "environment_id", value)

    @property
    @pulumi.getter(name="ipAllowList")
    def ip_allow_list(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['CidrBlockAndDescriptionArgs']]]]:
        return pulumi.get(self, "ip_allow_list")

    @ip_allow_list.setter
    def ip_allow_list(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['CidrBlockAndDescriptionArgs']]]]):
        pulumi.set(self, "ip_allow_list", value)

    @property
    @pulumi.getter(name="maxmemoryPolicy")
    def maxmemory_policy(self) -> Optional[pulumi.Input['MaxmemoryPolicy']]:
        """
        The eviction policy for the Redis instance
        """
        return pulumi.get(self, "maxmemory_policy")

    @maxmemory_policy.setter
    def maxmemory_policy(self, value: Optional[pulumi.Input['MaxmemoryPolicy']]):
        pulumi.set(self, "maxmemory_policy", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        The name of the Redis instance
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def region(self) -> Optional[pulumi.Input[str]]:
        """
        The region where the Redis instance is located
        """
        return pulumi.get(self, "region")

    @region.setter
    def region(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "region", value)


class Redis(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 environment_id: Optional[pulumi.Input[str]] = None,
                 ip_allow_list: Optional[pulumi.Input[Sequence[pulumi.Input[Union['CidrBlockAndDescriptionArgs', 'CidrBlockAndDescriptionArgsDict']]]]] = None,
                 maxmemory_policy: Optional[pulumi.Input['MaxmemoryPolicy']] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 owner_id: Optional[pulumi.Input[str]] = None,
                 plan: Optional[pulumi.Input['Plan']] = None,
                 region: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Input type for creating a Redis instance

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input['MaxmemoryPolicy'] maxmemory_policy: The eviction policy for the Redis instance
        :param pulumi.Input[str] name: The name of the Redis instance
        :param pulumi.Input[str] owner_id: The ID of the owner of the Redis instance
        :param pulumi.Input[str] region: The region where the Redis instance is located
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: RedisArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Input type for creating a Redis instance

        :param str resource_name: The name of the resource.
        :param RedisArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(RedisArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 environment_id: Optional[pulumi.Input[str]] = None,
                 ip_allow_list: Optional[pulumi.Input[Sequence[pulumi.Input[Union['CidrBlockAndDescriptionArgs', 'CidrBlockAndDescriptionArgsDict']]]]] = None,
                 maxmemory_policy: Optional[pulumi.Input['MaxmemoryPolicy']] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 owner_id: Optional[pulumi.Input[str]] = None,
                 plan: Optional[pulumi.Input['Plan']] = None,
                 region: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = RedisArgs.__new__(RedisArgs)

            __props__.__dict__["environment_id"] = environment_id
            __props__.__dict__["ip_allow_list"] = ip_allow_list
            __props__.__dict__["maxmemory_policy"] = maxmemory_policy
            __props__.__dict__["name"] = name
            if owner_id is None and not opts.urn:
                raise TypeError("Missing required property 'owner_id'")
            __props__.__dict__["owner_id"] = owner_id
            if plan is None and not opts.urn:
                raise TypeError("Missing required property 'plan'")
            __props__.__dict__["plan"] = plan
            __props__.__dict__["region"] = region
            __props__.__dict__["created_at"] = None
            __props__.__dict__["maintenance"] = None
            __props__.__dict__["options"] = None
            __props__.__dict__["owner"] = None
            __props__.__dict__["status"] = None
            __props__.__dict__["updated_at"] = None
            __props__.__dict__["version"] = None
        super(Redis, __self__).__init__(
            'render:redis:Redis',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None) -> 'Redis':
        """
        Get an existing Redis resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = RedisArgs.__new__(RedisArgs)

        __props__.__dict__["created_at"] = None
        __props__.__dict__["environment_id"] = None
        __props__.__dict__["ip_allow_list"] = None
        __props__.__dict__["maintenance"] = None
        __props__.__dict__["maxmemory_policy"] = None
        __props__.__dict__["name"] = None
        __props__.__dict__["options"] = None
        __props__.__dict__["owner"] = None
        __props__.__dict__["owner_id"] = None
        __props__.__dict__["plan"] = None
        __props__.__dict__["region"] = None
        __props__.__dict__["status"] = None
        __props__.__dict__["updated_at"] = None
        __props__.__dict__["version"] = None
        return Redis(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> pulumi.Output[str]:
        """
        The creation time of the Redis instance
        """
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter(name="environmentId")
    def environment_id(self) -> pulumi.Output[Optional[str]]:
        """
        The ID of the environment the Redis instance is associated with
        """
        return pulumi.get(self, "environment_id")

    @property
    @pulumi.getter(name="ipAllowList")
    def ip_allow_list(self) -> pulumi.Output[Sequence['outputs.CidrBlockAndDescription']]:
        """
        The IP allow list for the Redis instance
        """
        return pulumi.get(self, "ip_allow_list")

    @property
    @pulumi.getter
    def maintenance(self) -> pulumi.Output[Optional['outputs.MaintenanceProperties']]:
        return pulumi.get(self, "maintenance")

    @property
    @pulumi.getter(name="maxmemoryPolicy")
    def maxmemory_policy(self) -> pulumi.Output[Optional['MaxmemoryPolicy']]:
        """
        The eviction policy for the Redis instance
        """
        return pulumi.get(self, "maxmemory_policy")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        The name of the Redis instance
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def options(self) -> pulumi.Output['outputs.RedisOptions']:
        """
        Options for a Redis instance
        """
        return pulumi.get(self, "options")

    @property
    @pulumi.getter
    def owner(self) -> pulumi.Output['outputs.Owner']:
        return pulumi.get(self, "owner")

    @property
    @pulumi.getter(name="ownerId")
    def owner_id(self) -> pulumi.Output[str]:
        """
        The ID of the owner of the Redis instance
        """
        return pulumi.get(self, "owner_id")

    @property
    @pulumi.getter
    def plan(self) -> pulumi.Output['Plan']:
        return pulumi.get(self, "plan")

    @property
    @pulumi.getter
    def region(self) -> pulumi.Output['Region']:
        """
        Defaults to "oregon"
        """
        return pulumi.get(self, "region")

    @property
    @pulumi.getter
    def status(self) -> pulumi.Output['Status']:
        return pulumi.get(self, "status")

    @property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> pulumi.Output[str]:
        """
        The last updated time of the Redis instance
        """
        return pulumi.get(self, "updated_at")

    @property
    @pulumi.getter
    def version(self) -> pulumi.Output[str]:
        """
        The version of Redis
        """
        return pulumi.get(self, "version")




# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *
from ._inputs import *

__all__ = ['EnvGroupArgs', 'EnvGroup']

@pulumi.input_type
class EnvGroupArgs:
    def __init__(__self__, *,
                 env_vars: pulumi.Input[Sequence[pulumi.Input['EnvVarInputArgs']]],
                 owner_id: pulumi.Input[str],
                 environment_id: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 secret_files: Optional[pulumi.Input[Sequence[pulumi.Input['SecretFileInputArgs']]]] = None,
                 service_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None):
        """
        The set of arguments for constructing a EnvGroup resource.
        """
        pulumi.set(__self__, "env_vars", env_vars)
        pulumi.set(__self__, "owner_id", owner_id)
        if environment_id is not None:
            pulumi.set(__self__, "environment_id", environment_id)
        if name is not None:
            pulumi.set(__self__, "name", name)
        if secret_files is not None:
            pulumi.set(__self__, "secret_files", secret_files)
        if service_ids is not None:
            pulumi.set(__self__, "service_ids", service_ids)

    @property
    @pulumi.getter(name="envVars")
    def env_vars(self) -> pulumi.Input[Sequence[pulumi.Input['EnvVarInputArgs']]]:
        return pulumi.get(self, "env_vars")

    @env_vars.setter
    def env_vars(self, value: pulumi.Input[Sequence[pulumi.Input['EnvVarInputArgs']]]):
        pulumi.set(self, "env_vars", value)

    @property
    @pulumi.getter(name="ownerId")
    def owner_id(self) -> pulumi.Input[str]:
        return pulumi.get(self, "owner_id")

    @owner_id.setter
    def owner_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "owner_id", value)

    @property
    @pulumi.getter(name="environmentId")
    def environment_id(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "environment_id")

    @environment_id.setter
    def environment_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "environment_id", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="secretFiles")
    def secret_files(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['SecretFileInputArgs']]]]:
        return pulumi.get(self, "secret_files")

    @secret_files.setter
    def secret_files(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['SecretFileInputArgs']]]]):
        pulumi.set(self, "secret_files", value)

    @property
    @pulumi.getter(name="serviceIds")
    def service_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        return pulumi.get(self, "service_ids")

    @service_ids.setter
    def service_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "service_ids", value)


class EnvGroup(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 env_vars: Optional[pulumi.Input[Sequence[pulumi.Input[Union['EnvVarInputArgs', 'EnvVarInputArgsDict']]]]] = None,
                 environment_id: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 owner_id: Optional[pulumi.Input[str]] = None,
                 secret_files: Optional[pulumi.Input[Sequence[pulumi.Input[Union['SecretFileInputArgs', 'SecretFileInputArgsDict']]]]] = None,
                 service_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 __props__=None):
        """
        Create a EnvGroup resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: EnvGroupArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Create a EnvGroup resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param EnvGroupArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(EnvGroupArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 env_vars: Optional[pulumi.Input[Sequence[pulumi.Input[Union['EnvVarInputArgs', 'EnvVarInputArgsDict']]]]] = None,
                 environment_id: Optional[pulumi.Input[str]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 owner_id: Optional[pulumi.Input[str]] = None,
                 secret_files: Optional[pulumi.Input[Sequence[pulumi.Input[Union['SecretFileInputArgs', 'SecretFileInputArgsDict']]]]] = None,
                 service_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = EnvGroupArgs.__new__(EnvGroupArgs)

            if env_vars is None and not opts.urn:
                raise TypeError("Missing required property 'env_vars'")
            __props__.__dict__["env_vars"] = env_vars
            __props__.__dict__["environment_id"] = environment_id
            __props__.__dict__["name"] = name
            if owner_id is None and not opts.urn:
                raise TypeError("Missing required property 'owner_id'")
            __props__.__dict__["owner_id"] = owner_id
            __props__.__dict__["secret_files"] = secret_files
            __props__.__dict__["service_ids"] = service_ids
            __props__.__dict__["created_at"] = None
            __props__.__dict__["service_links"] = None
            __props__.__dict__["updated_at"] = None
        super(EnvGroup, __self__).__init__(
            'render:env-groups:EnvGroup',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None) -> 'EnvGroup':
        """
        Get an existing EnvGroup resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = EnvGroupArgs.__new__(EnvGroupArgs)

        __props__.__dict__["created_at"] = None
        __props__.__dict__["env_vars"] = None
        __props__.__dict__["environment_id"] = None
        __props__.__dict__["name"] = None
        __props__.__dict__["owner_id"] = None
        __props__.__dict__["secret_files"] = None
        __props__.__dict__["service_ids"] = None
        __props__.__dict__["service_links"] = None
        __props__.__dict__["updated_at"] = None
        return EnvGroup(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter(name="envVars")
    def env_vars(self) -> pulumi.Output[Sequence['outputs.EnvVar']]:
        return pulumi.get(self, "env_vars")

    @property
    @pulumi.getter(name="environmentId")
    def environment_id(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "environment_id")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="ownerId")
    def owner_id(self) -> pulumi.Output[str]:
        return pulumi.get(self, "owner_id")

    @property
    @pulumi.getter(name="secretFiles")
    def secret_files(self) -> pulumi.Output[Optional[Sequence['outputs.SecretFile']]]:
        return pulumi.get(self, "secret_files")

    @property
    @pulumi.getter(name="serviceIds")
    def service_ids(self) -> pulumi.Output[Optional[Sequence[str]]]:
        return pulumi.get(self, "service_ids")

    @property
    @pulumi.getter(name="serviceLinks")
    def service_links(self) -> pulumi.Output[Optional[Sequence['outputs.ServiceLink']]]:
        """
        List of serviceIds linked to the envGroup
        """
        return pulumi.get(self, "service_links")

    @property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> pulumi.Output[Optional[str]]:
        return pulumi.get(self, "updated_at")




# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload, Awaitable
from .. import _utilities
from ._enums import *

__all__ = [
    'GetPostgresRecoveryInfoProperties',
    'AwaitableGetPostgresRecoveryInfoProperties',
    'get_postgres_recovery_info',
    'get_postgres_recovery_info_output',
]

@pulumi.output_type
class GetPostgresRecoveryInfoProperties:
    def __init__(__self__, recovery_status=None, starts_at=None):
        if recovery_status and not isinstance(recovery_status, str):
            raise TypeError("Expected argument 'recovery_status' to be a str")
        pulumi.set(__self__, "recovery_status", recovery_status)
        if starts_at and not isinstance(starts_at, str):
            raise TypeError("Expected argument 'starts_at' to be a str")
        pulumi.set(__self__, "starts_at", starts_at)

    @property
    @pulumi.getter(name="recoveryStatus")
    def recovery_status(self) -> 'GetPostgresRecoveryInfoPropertiesRecoveryStatus':
        """
        Availability of point-in-time recovery.
        """
        return pulumi.get(self, "recovery_status")

    @property
    @pulumi.getter(name="startsAt")
    def starts_at(self) -> Optional[str]:
        return pulumi.get(self, "starts_at")


class AwaitableGetPostgresRecoveryInfoProperties(GetPostgresRecoveryInfoProperties):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetPostgresRecoveryInfoProperties(
            recovery_status=self.recovery_status,
            starts_at=self.starts_at)


def get_postgres_recovery_info(postgres_id: Optional[str] = None,
                               opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetPostgresRecoveryInfoProperties:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['postgresId'] = postgres_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:postgres:getPostgresRecoveryInfo', __args__, opts=opts, typ=GetPostgresRecoveryInfoProperties).value

    return AwaitableGetPostgresRecoveryInfoProperties(
        recovery_status=pulumi.get(__ret__, 'recovery_status'),
        starts_at=pulumi.get(__ret__, 'starts_at'))


@_utilities.lift_output_func(get_postgres_recovery_info)
def get_postgres_recovery_info_output(postgres_id: Optional[pulumi.Input[str]] = None,
                                      opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetPostgresRecoveryInfoProperties]:
    """
    Use this data source to access information about an existing resource.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from enum import Enum

__all__ = [
    'BackgroundWorkerDetailsCreateEnv',
    'BackgroundWorkerDetailsCreatePlan',
    'BackgroundWorkerDetailsCreatePullRequestPreviewsEnabled',
    'BackgroundWorkerDetailsCreateRegion',
    'BackgroundWorkerDetailsCreateRuntime',
    'BackgroundWorkerDetailsOutputBuildPlan',
    'BackgroundWorkerDetailsOutputEnv',
    'BackgroundWorkerDetailsOutputPlan',
    'BackgroundWorkerDetailsOutputPullRequestPreviewsEnabled',
    'BackgroundWorkerDetailsOutputRegion',
    'CancelJobStatus',
    'ClearCache',
    'CronJobDetailsCreateEnv',
    'CronJobDetailsCreatePlan',
    'CronJobDetailsCreateRegion',
    'CronJobDetailsCreateRuntime',
    'CronJobDetailsOutputBuildPlan',
    'CronJobDetailsOutputEnv',
    'CronJobDetailsOutputPlan',
    'CronJobDetailsOutputRegion',
    'CustomDomainDomainType',
    'CustomDomainVerificationStatus',
    'DeployStatus',
    'DeployTrigger',
    'JobStatus',
    'Paths1services17BserviceId7D1jobscreateresponses200contentapplication1jsonschemaStatus',
    'Plan',
    'PrivateServiceDetailsCreateEnv',
    'PrivateServiceDetailsCreatePlan',
    'PrivateServiceDetailsCreatePullRequestPreviewsEnabled',
    'PrivateServiceDetailsCreateRegion',
    'PrivateServiceDetailsCreateRuntime',
    'PrivateServiceDetailsOutputBuildPlan',
    'PrivateServiceDetailsOutputEnv',
    'PrivateServiceDetailsOutputPlan',
    'PrivateServiceDetailsOutputPullRequestPreviewsEnabled',
    'PrivateServiceDetailsOutputRegion',
    'RegistryCredentialRegistry',
    'RouteCreateType',
    'RouteType',
    'ServerPortProtocol',
    'ServiceAutoDeploy',
    'ServiceCreateAutoDeploy',
    'ServiceNotifyOnFail',
    'ServiceSuspended',
    'ServiceSuspendersItem',
    'StaticSiteDetailsCreatePullRequestPreviewsEnabled',
    'StaticSiteDetailsOutputBuildPlan',
    'StaticSiteDetailsOutputPullRequestPreviewsEnabled',
    'Status',
    'Trigger',
    'Type',
    'WebServiceDetailsCreateEnv',
    'WebServiceDetailsCreatePlan',
    'WebServiceDetailsCreatePullRequestPreviewsEnabled',
    'WebServiceDetailsCreateRegion',
    'WebServiceDetailsCreateRuntime',
    'WebServiceDetailsOutputBuildPlan',
    'WebServiceDetailsOutputEnv',
    'WebServiceDetailsOutputPlan',
    'WebServiceDetailsOutputPullRequestPreviewsEnabled',
    'WebServiceDetailsOutputRegion',
]


class BackgroundWorkerDetailsCreateEnv(str, Enum):
    """
    This field has been deprecated, runtime should be used in its place.
    """
    DOCKER = "docker"
    ELIXIR = "elixir"
    GO = "go"
    NODE = "node"
    PYTHON = "python"
    RUBY = "ruby"
    RUST = "rust"
    IMAGE = "image"


class BackgroundWorkerDetailsCreatePlan(str, Enum):
    """
    Defaults to "starter"
    """
    STARTER = "starter"
    STANDARD = "standard"
    PRO = "pro"
    PRO_PLUS = "pro_plus"
    PRO_MAX = "pro_max"
    PRO_ULTRA = "pro_ultra"


class BackgroundWorkerDetailsCreatePullRequestPreviewsEnabled(str, Enum):
    """
    Defaults to "no"
    """
    YES = "yes"
    NO = "no"


class BackgroundWorkerDetailsCreateRegion(str, Enum):
    """
    Defaults to "oregon"
    """
    FRANKFURT = "frankfurt"
    OREGON = "oregon"
    OHIO = "ohio"
    SINGAPORE = "singapore"
    VIRGINIA = "virginia"


class BackgroundWorkerDetailsCreateRuntime(str, Enum):
    """
    Runtime
    """
    DOCKER = "docker"
    ELIXIR = "elixir"
    GO = "go"
    NODE = "node"
    PYTHON = "python"
    RUBY = "ruby"
    RUST = "rust"
    IMAGE = "image"


class BackgroundWorkerDetailsOutputBuildPlan(str, Enum):
    STARTER = "starter"
    PERFORMANCE = "performance"


class BackgroundWorkerDetailsOutputEnv(str, Enum):
    """
    Runtime
    """
    DOCKER = "docker"
    ELIXIR = "elixir"
    GO = "go"
    NODE = "node"
    PYTHON = "python"
    RUBY = "ruby"
    RUST = "rust"
    IMAGE = "image"


class BackgroundWorkerDetailsOutputPlan(str, Enum):
    """
    The instance type to use for the preview instance. Note that base services with any paid instance type can't create preview instances with the `free` instance type.
    """
    STARTER = "starter"
    STARTER_PLUS = "starter_plus"
    STANDARD = "standard"
    STANDARD_PLUS = "standard_plus"
    PRO = "pro"
    PRO_PLUS = "pro_plus"
    PRO_MAX = "pro_max"
    PRO_ULTRA = "pro_ultra"
    FREE = "free"
    CUSTOM = "custom"


class BackgroundWorkerDetailsOutputPullRequestPreviewsEnabled(str, Enum):
    """
    Defaults to "no"
    """
    YES = "yes"
    NO = "no"


class BackgroundWorkerDetailsOutputRegion(str, Enum):
    """
    Defaults to "oregon"
    """
    FRANKFURT = "frankfurt"
    OREGON = "oregon"
    OHIO = "ohio"
    SINGAPORE = "singapore"
    VIRGINIA = "virginia"


class CancelJobStatus(str, Enum):
    SUCCEEDED = "succeeded"
    FAILED = "failed"
    CANCELED = "canceled"


class ClearCache(str, Enum):
    """
    If `clear`, Render clears the service's build cache before deploying. This can be useful if you're experiencing issues with your build.
    """
    CLEAR = "clear"
    DO_NOT_CLEAR = "do_not_clear"


class CronJobDetailsCreateEnv(str, Enum):
    """
    This field has been deprecated, runtime should be used in its place.
    """
    DOCKER = "docker"
    ELIXIR = "elixir"
    GO = "go"
    NODE = "node"
    PYTHON = "python"
    RUBY = "ruby"
    RUST = "rust"
    IMAGE = "image"


class CronJobDetailsCreatePlan(str, Enum):
    """
    Defaults to "starter"
    """
    STARTER = "starter"
    STANDARD = "standard"
    PRO = "pro"
    PRO_PLUS = "pro_plus"
    PRO_MAX = "pro_max"
    PRO_ULTRA = "pro_ultra"


class CronJobDetailsCreateRegion(str, Enum):
    """
    Defaults to "oregon"
    """
    FRANKFURT = "frankfurt"
    OREGON = "oregon"
    OHIO = "ohio"
    SINGAPORE = "singapore"
    VIRGINIA = "virginia"


class CronJobDetailsCreateRuntime(str, Enum):
    """
    Runtime
    """
    DOCKER = "docker"
    ELIXIR = "elixir"
    GO = "go"
    NODE = "node"
    PYTHON = "python"
    RUBY = "ruby"
    RUST = "rust"
    IMAGE = "image"


class CronJobDetailsOutputBuildPlan(str, Enum):
    STARTER = "starter"
    PERFORMANCE = "performance"


class CronJobDetailsOutputEnv(str, Enum):
    """
    Runtime
    """
    DOCKER = "docker"
    ELIXIR = "elixir"
    GO = "go"
    NODE = "node"
    PYTHON = "python"
    RUBY = "ruby"
    RUST = "rust"
    IMAGE = "image"


class CronJobDetailsOutputPlan(str, Enum):
    """
    The instance type to use for the preview instance. Note that base services with any paid instance type can't create preview instances with the `free` instance type.
    """
    STARTER = "starter"
    STARTER_PLUS = "starter_plus"
    STANDARD = "standard"
    STANDARD_PLUS = "standard_plus"
    PRO = "pro"
    PRO_PLUS = "pro_plus"
    PRO_MAX = "pro_max"
    PRO_ULTRA = "pro_ultra"
    FREE = "free"
    CUSTOM = "custom"


class CronJobDetailsOutputRegion(str, Enum):
    """
    Defaults to "oregon"
    """
    FRANKFURT = "frankfurt"
    OREGON = "oregon"
    OHIO = "ohio"
    SINGAPORE = "singapore"
    VIRGINIA = "virginia"


class CustomDomainDomainType(str, Enum):
    APEX = "apex"
    SUBDOMAIN = "subdomain"


class CustomDomainVerificationStatus(str, Enum):
    VERIFIED = "verified"
    UNVERIFIED = "unverified"


class DeployStatus(str, Enum):
    CREATED = "created"
    BUILD_IN_PROGRESS = "build_in_progress"
    UPDATE_IN_PROGRESS = "update_in_progress"
    LIVE = "live"
    DEACTIVATED = "deactivated"
    BUILD_FAILED = "build_failed"
    UPDATE_FAILED = "update_failed"
    CANCELED = "canceled"
    PRE_DEPLOY_IN_PROGRESS = "pre_deploy_in_progress"
    PRE_DEPLOY_FAILED = "pre_deploy_failed"


class DeployTrigger(str, Enum):
    API = "api"
    BLUEPRINT_SYNC = "blueprint_sync"
    DEPLOY_HOOK = "deploy_hook"
    DEPLOYED_BY_RENDER = "deployed_by_render"
    MANUAL = "manual"
    OTHER = "other"
    NEW_COMMIT = "new_commit"
    ROLLBACK = "rollback"
    SERVICE_RESUMED = "service_resumed"
    SERVICE_UPDATED = "service_updated"


class JobStatus(str, Enum):
    SUCCEEDED = "succeeded"
    FAILED = "failed"
    CANCELED = "canceled"


class Paths1services17BserviceId7D1jobscreateresponses200contentapplication1jsonschemaStatus(str, Enum):
    SUCCEEDED = "succeeded"
    FAILED = "failed"
    CANCELED = "canceled"


class Plan(str, Enum):
    """
    The instance type to use for the preview instance. Note that base services with any paid instance type can't create preview instances with the `free` instance type.
    """
    STARTER = "starter"
    STARTER_PLUS = "starter_plus"
    STANDARD = "standard"
    STANDARD_PLUS = "standard_plus"
    PRO = "pro"
    PRO_PLUS = "pro_plus"
    PRO_MAX = "pro_max"
    PRO_ULTRA = "pro_ultra"
    FREE = "free"
    CUSTOM = "custom"


class PrivateServiceDetailsCreateEnv(str, Enum):
    """
    This field has been deprecated, runtime should be used in its place.
    """
    DOCKER = "docker"
    ELIXIR = "elixir"
    GO = "go"
    NODE = "node"
    PYTHON = "python"
    RUBY = "ruby"
    RUST = "rust"
    IMAGE = "image"


class PrivateServiceDetailsCreatePlan(str, Enum):
    """
    Defaults to "starter"
    """
    STARTER = "starter"
    STANDARD = "standard"
    PRO = "pro"
    PRO_PLUS = "pro_plus"
    PRO_MAX = "pro_max"
    PRO_ULTRA = "pro_ultra"


class PrivateServiceDetailsCreatePullRequestPreviewsEnabled(str, Enum):
    """
    Defaults to "no"
    """
    YES = "yes"
    NO = "no"


class PrivateServiceDetailsCreateRegion(str, Enum):
    """
    Defaults to "oregon"
    """
    FRANKFURT = "frankfurt"
    OREGON = "oregon"
    OHIO = "ohio"
    SINGAPORE = "singapore"
    VIRGINIA = "virginia"


class PrivateServiceDetailsCreateRuntime(str, Enum):
    """
    Runtime
    """
    DOCKER = "docker"
    ELIXIR = "elixir"
    GO = "go"
    NODE = "node"
    PYTHON = "python"
    RUBY = "ruby"
    RUST = "rust"
    IMAGE = "image"


class PrivateServiceDetailsOutputBuildPlan(str, Enum):
    STARTER = "starter"
    PERFORMANCE = "performance"


class PrivateServiceDetailsOutputEnv(str, Enum):
    """
    Runtime
    """
    DOCKER = "docker"
    ELIXIR = "elixir"
    GO = "go"
    NODE = "node"
    PYTHON = "python"
    RUBY = "ruby"
    RUST = "rust"
    IMAGE = "image"


class PrivateServiceDetailsOutputPlan(str, Enum):
    """
    The instance type to use for the preview instance. Note that base services with any paid instance type can't create preview instances with the `free` instance type.
    """
    STARTER = "starter"
    STARTER_PLUS = "starter_plus"
    STANDARD = "standard"
    STANDARD_PLUS = "standard_plus"
    PRO = "pro"
    PRO_PLUS = "pro_plus"
    PRO_MAX = "pro_max"
    PRO_ULTRA = "pro_ultra"
    FREE = "free"
    CUSTOM = "custom"


class PrivateServiceDetailsOutputPullRequestPreviewsEnabled(str, Enum):
    """
    Defaults to "no"
    """
    YES = "yes"
    NO = "no"


class PrivateServiceDetailsOutputRegion(str, Enum):
    """
    Defaults to "oregon"
    """
    FRANKFURT = "frankfurt"
    OREGON = "oregon"
    OHIO = "ohio"
    SINGAPORE = "singapore"
    VIRGINIA = "virginia"


class RegistryCredentialRegistry(str, Enum):
    """
    The registry to use this credential with
    """
    GITHUB = "GITHUB"
    GITLAB = "GITLAB"
    DOCKER = "DOCKER"
    GOOGLE_ARTIFACT = "GOOGLE_ARTIFACT"


class RouteCreateType(str, Enum):
    REDIRECT = "redirect"
    REWRITE = "rewrite"


class RouteType(str, Enum):
    REDIRECT = "redirect"
    REWRITE = "rewrite"


class ServerPortProtocol(str, Enum):
    TCP = "TCP"
    UDP = "UDP"


class ServiceAutoDeploy(str, Enum):
    YES = "yes"
    NO = "no"


class ServiceCreateAutoDeploy(str, Enum):
    YES = "yes"
    NO = "no"


class ServiceNotifyOnFail(str, Enum):
    DEFAULT = "default"
    NOTIFY = "notify"
    IGNORE = "ignore"


class ServiceSuspended(str, Enum):
    SUSPENDED = "suspended"
    NOT_SUSPENDED = "not_suspended"


class ServiceSuspendersItem(str, Enum):
    ADMIN = "admin"
    BILLING = "billing"
    USER = "user"
    PARENT_SERVICE = "parent_service"
    UNKNOWN = "unknown"


class StaticSiteDetailsCreatePullRequestPreviewsEnabled(str, Enum):
    """
    Defaults to "no"
    """
    YES = "yes"
    NO = "no"


class StaticSiteDetailsOutputBuildPlan(str, Enum):
    STARTER = "starter"
    PERFORMANCE = "performance"


class StaticSiteDetailsOutputPullRequestPreviewsEnabled(str, Enum):
    """
    Defaults to "no"
    """
    YES = "yes"
    NO = "no"


class Status(str, Enum):
    CREATED = "created"
    BUILD_IN_PROGRESS = "build_in_progress"
    UPDATE_IN_PROGRESS = "update_in_progress"
    LIVE = "live"
    DEACTIVATED = "deactivated"
    BUILD_FAILED = "build_failed"
    UPDATE_FAILED = "update_failed"
    CANCELED = "canceled"
    PRE_DEPLOY_IN_PROGRESS = "pre_deploy_in_progress"
    PRE_DEPLOY_FAILED = "pre_deploy_failed"


class Trigger(str, Enum):
    API = "api"
    BLUEPRINT_SYNC = "blueprint_sync"
    DEPLOY_HOOK = "deploy_hook"
    DEPLOYED_BY_RENDER = "deployed_by_render"
    MANUAL = "manual"
    OTHER = "other"
    NEW_COMMIT = "new_commit"
    ROLLBACK = "rollback"
    SERVICE_RESUMED = "service_resumed"
    SERVICE_UPDATED = "service_updated"


class Type(str, Enum):
    REDIRECT = "redirect"
    REWRITE = "rewrite"


class WebServiceDetailsCreateEnv(str, Enum):
    """
    This field has been deprecated, runtime should be used in its place.
    """
    DOCKER = "docker"
    ELIXIR = "elixir"
    GO = "go"
    NODE = "node"
    PYTHON = "python"
    RUBY = "ruby"
    RUST = "rust"
    IMAGE = "image"


class WebServiceDetailsCreatePlan(str, Enum):
    """
    Defaults to "starter"
    """
    STARTER = "starter"
    STANDARD = "standard"
    PRO = "pro"
    PRO_PLUS = "pro_plus"
    PRO_MAX = "pro_max"
    PRO_ULTRA = "pro_ultra"


class WebServiceDetailsCreatePullRequestPreviewsEnabled(str, Enum):
    """
    Defaults to "no"
    """
    YES = "yes"
    NO = "no"


class WebServiceDetailsCreateRegion(str, Enum):
    """
    Defaults to "oregon"
    """
    FRANKFURT = "frankfurt"
    OREGON = "oregon"
    OHIO = "ohio"
    SINGAPORE = "singapore"
    VIRGINIA = "virginia"


class WebServiceDetailsCreateRuntime(str, Enum):
    """
    Runtime
    """
    DOCKER = "docker"
    ELIXIR = "elixir"
    GO = "go"
    NODE = "node"
    PYTHON = "python"
    RUBY = "ruby"
    RUST = "rust"
    IMAGE = "image"


class WebServiceDetailsOutputBuildPlan(str, Enum):
    STARTER = "starter"
    PERFORMANCE = "performance"


class WebServiceDetailsOutputEnv(str, Enum):
    """
    Runtime
    """
    DOCKER = "docker"
    ELIXIR = "elixir"
    GO = "go"
    NODE = "node"
    PYTHON = "python"
    RUBY = "ruby"
    RUST = "rust"
    IMAGE = "image"


class WebServiceDetailsOutputPlan(str, Enum):
    """
    The instance type to use for the preview instance. Note that base services with any paid instance type can't create preview instances with the `free` instance type.
    """
    STARTER = "starter"
    STARTER_PLUS = "starter_plus"
    STANDARD = "standard"
    STANDARD_PLUS = "standard_plus"
    PRO = "pro"
    PRO_PLUS = "pro_plus"
    PRO_MAX = "pro_max"
    PRO_ULTRA = "pro_ultra"
    FREE = "free"
    CUSTOM = "custom"


class WebServiceDetailsOutputPullRequestPreviewsEnabled(str, Enum):
    """
    Defaults to "no"
    """
    YES = "yes"
    NO = "no"


class WebServiceDetailsOutputRegion(str, Enum):
    """
    Defaults to "oregon"
    """
    FRANKFURT = "frankfurt"
    OREGON = "oregon"
    OHIO = "ohio"
    SINGAPORE = "singapore"
    VIRGINIA = "virginia"



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload, Awaitable
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'GetBlueprintProperties',
    'AwaitableGetBlueprintProperties',
    'get_blueprint',
    'get_blueprint_output',
]

@pulumi.output_type
class GetBlueprintProperties:
    def __init__(__self__, auto_sync=None, branch=None, id=None, last_sync=None, name=None, repo=None, resources=None, status=None):
        if auto_sync and not isinstance(auto_sync, bool):
            raise TypeError("Expected argument 'auto_sync' to be a bool")
        pulumi.set(__self__, "auto_sync", auto_sync)
        if branch and not isinstance(branch, str):
            raise TypeError("Expected argument 'branch' to be a str")
        pulumi.set(__self__, "branch", branch)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if last_sync and not isinstance(last_sync, str):
            raise TypeError("Expected argument 'last_sync' to be a str")
        pulumi.set(__self__, "last_sync", last_sync)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if repo and not isinstance(repo, str):
            raise TypeError("Expected argument 'repo' to be a str")
        pulumi.set(__self__, "repo", repo)
        if resources and not isinstance(resources, list):
            raise TypeError("Expected argument 'resources' to be a list")
        pulumi.set(__self__, "resources", resources)
        if status and not isinstance(status, str):
            raise TypeError("Expected argument 'status' to be a str")
        pulumi.set(__self__, "status", status)

    @property
    @pulumi.getter(name="autoSync")
    def auto_sync(self) -> bool:
        """
        Automatically sync changes to render.yaml
        """
        return pulumi.get(self, "auto_sync")

    @property
    @pulumi.getter
    def branch(self) -> str:
        return pulumi.get(self, "branch")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="lastSync")
    def last_sync(self) -> Optional[str]:
        return pulumi.get(self, "last_sync")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def repo(self) -> str:
        return pulumi.get(self, "repo")

    @property
    @pulumi.getter
    def resources(self) -> Sequence['outputs.GetBlueprintPropertiesResourcesItemProperties']:
        return pulumi.get(self, "resources")

    @property
    @pulumi.getter
    def status(self) -> 'GetBlueprintPropertiesStatus':
        return pulumi.get(self, "status")


class AwaitableGetBlueprintProperties(GetBlueprintProperties):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetBlueprintProperties(
            auto_sync=self.auto_sync,
            branch=self.branch,
            id=self.id,
            last_sync=self.last_sync,
            name=self.name,
            repo=self.repo,
            resources=self.resources,
            status=self.status)


def get_blueprint(blueprint_id: Optional[str] = None,
                  opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetBlueprintProperties:
    """
    Use this data source to access information about an existing resource.

    :param str blueprint_id: The ID of the blueprint
    """
    __args__ = dict()
    __args__['blueprintId'] = blueprint_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:blueprints:getBlueprint', __args__, opts=opts, typ=GetBlueprintProperties).value

    return AwaitableGetBlueprintProperties(
        auto_sync=pulumi.get(__ret__, 'auto_sync'),
        branch=pulumi.get(__ret__, 'branch'),
        id=pulumi.get(__ret__, 'id'),
        last_sync=pulumi.get(__ret__, 'last_sync'),
        name=pulumi.get(__ret__, 'name'),
        repo=pulumi.get(__ret__, 'repo'),
        resources=pulumi.get(__ret__, 'resources'),
        status=pulumi.get(__ret__, 'status'))


@_utilities.lift_output_func(get_blueprint)
def get_blueprint_output(blueprint_id: Optional[pulumi.Input[str]] = None,
                         opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetBlueprintProperties]:
    """
    Use this data source to access information about an existing resource.

    :param str blueprint_id: The ID of the blueprint
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload, Awaitable
from .. import _utilities
from ._enums import *

__all__ = [
    'Job',
    'AwaitableJob',
    'get_job',
    'get_job_output',
]

@pulumi.output_type
class Job:
    def __init__(__self__, created_at=None, finished_at=None, id=None, plan_id=None, service_id=None, start_command=None, started_at=None, status=None):
        if created_at and not isinstance(created_at, str):
            raise TypeError("Expected argument 'created_at' to be a str")
        pulumi.set(__self__, "created_at", created_at)
        if finished_at and not isinstance(finished_at, str):
            raise TypeError("Expected argument 'finished_at' to be a str")
        pulumi.set(__self__, "finished_at", finished_at)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if plan_id and not isinstance(plan_id, str):
            raise TypeError("Expected argument 'plan_id' to be a str")
        pulumi.set(__self__, "plan_id", plan_id)
        if service_id and not isinstance(service_id, str):
            raise TypeError("Expected argument 'service_id' to be a str")
        pulumi.set(__self__, "service_id", service_id)
        if start_command and not isinstance(start_command, str):
            raise TypeError("Expected argument 'start_command' to be a str")
        pulumi.set(__self__, "start_command", start_command)
        if started_at and not isinstance(started_at, str):
            raise TypeError("Expected argument 'started_at' to be a str")
        pulumi.set(__self__, "started_at", started_at)
        if status and not isinstance(status, str):
            raise TypeError("Expected argument 'status' to be a str")
        pulumi.set(__self__, "status", status)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> str:
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter(name="finishedAt")
    def finished_at(self) -> Optional[str]:
        return pulumi.get(self, "finished_at")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="planId")
    def plan_id(self) -> str:
        return pulumi.get(self, "plan_id")

    @property
    @pulumi.getter(name="serviceId")
    def service_id(self) -> str:
        return pulumi.get(self, "service_id")

    @property
    @pulumi.getter(name="startCommand")
    def start_command(self) -> str:
        return pulumi.get(self, "start_command")

    @property
    @pulumi.getter(name="startedAt")
    def started_at(self) -> Optional[str]:
        return pulumi.get(self, "started_at")

    @property
    @pulumi.getter
    def status(self) -> Optional['JobStatus']:
        return pulumi.get(self, "status")


class AwaitableJob(Job):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return Job(
            created_at=self.created_at,
            finished_at=self.finished_at,
            id=self.id,
            plan_id=self.plan_id,
            service_id=self.service_id,
            start_command=self.start_command,
            started_at=self.started_at,
            status=self.status)


def get_job(job_id: Optional[str] = None,
            service_id: Optional[str] = None,
            opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableJob:
    """
    Use this data source to access information about an existing resource.

    :param str job_id: The ID of the job
    :param str service_id: The ID of the service
    """
    __args__ = dict()
    __args__['jobId'] = job_id
    __args__['serviceId'] = service_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:services:getJob', __args__, opts=opts, typ=Job).value

    return AwaitableJob(
        created_at=pulumi.get(__ret__, 'created_at'),
        finished_at=pulumi.get(__ret__, 'finished_at'),
        id=pulumi.get(__ret__, 'id'),
        plan_id=pulumi.get(__ret__, 'plan_id'),
        service_id=pulumi.get(__ret__, 'service_id'),
        start_command=pulumi.get(__ret__, 'start_command'),
        started_at=pulumi.get(__ret__, 'started_at'),
        status=pulumi.get(__ret__, 'status'))


@_utilities.lift_output_func(get_job)
def get_job_output(job_id: Optional[pulumi.Input[str]] = None,
                   service_id: Optional[pulumi.Input[str]] = None,
                   opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[Job]:
    """
    Use this data source to access information about an existing resource.

    :param str job_id: The ID of the job
    :param str service_id: The ID of the service
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from enum import Enum

__all__ = [
    'EnvironmentProtectedStatus',
    'ProtectedStatus',
]


class EnvironmentProtectedStatus(str, Enum):
    """
    Indicates whether an environment is `unprotected` or `protected`. Only admin users can perform destructive actions in `protected` environments.
    """
    UNPROTECTED = "unprotected"
    PROTECTED = "protected"


class ProtectedStatus(str, Enum):
    """
    Indicates whether an environment is `unprotected` or `protected`. Only admin users can perform destructive actions in `protected` environments.
    """
    UNPROTECTED = "unprotected"
    PROTECTED = "protected"



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload, Awaitable
from .. import _utilities

__all__ = [
    'DiskWithCursorpropertiesdisk',
    'AwaitableDiskWithCursorpropertiesdisk',
    'get_disk',
    'get_disk_output',
]

@pulumi.output_type
class DiskWithCursorpropertiesdisk:
    def __init__(__self__, created_at=None, id=None, mount_path=None, name=None, service_id=None, size_gb=None, updated_at=None):
        if created_at and not isinstance(created_at, str):
            raise TypeError("Expected argument 'created_at' to be a str")
        pulumi.set(__self__, "created_at", created_at)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if mount_path and not isinstance(mount_path, str):
            raise TypeError("Expected argument 'mount_path' to be a str")
        pulumi.set(__self__, "mount_path", mount_path)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if service_id and not isinstance(service_id, str):
            raise TypeError("Expected argument 'service_id' to be a str")
        pulumi.set(__self__, "service_id", service_id)
        if size_gb and not isinstance(size_gb, int):
            raise TypeError("Expected argument 'size_gb' to be a int")
        pulumi.set(__self__, "size_gb", size_gb)
        if updated_at and not isinstance(updated_at, str):
            raise TypeError("Expected argument 'updated_at' to be a str")
        pulumi.set(__self__, "updated_at", updated_at)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> str:
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="mountPath")
    def mount_path(self) -> str:
        return pulumi.get(self, "mount_path")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="serviceId")
    def service_id(self) -> Optional[str]:
        return pulumi.get(self, "service_id")

    @property
    @pulumi.getter(name="sizeGB")
    def size_gb(self) -> int:
        return pulumi.get(self, "size_gb")

    @property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> str:
        return pulumi.get(self, "updated_at")


class AwaitableDiskWithCursorpropertiesdisk(DiskWithCursorpropertiesdisk):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return DiskWithCursorpropertiesdisk(
            created_at=self.created_at,
            id=self.id,
            mount_path=self.mount_path,
            name=self.name,
            service_id=self.service_id,
            size_gb=self.size_gb,
            updated_at=self.updated_at)


def get_disk(disk_id: Optional[str] = None,
             opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableDiskWithCursorpropertiesdisk:
    """
    Use this data source to access information about an existing resource.

    :param str disk_id: The ID of the disk
    """
    __args__ = dict()
    __args__['diskId'] = disk_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:disks:getDisk', __args__, opts=opts, typ=DiskWithCursorpropertiesdisk).value

    return AwaitableDiskWithCursorpropertiesdisk(
        created_at=pulumi.get(__ret__, 'created_at'),
        id=pulumi.get(__ret__, 'id'),
        mount_path=pulumi.get(__ret__, 'mount_path'),
        name=pulumi.get(__ret__, 'name'),
        service_id=pulumi.get(__ret__, 'service_id'),
        size_gb=pulumi.get(__ret__, 'size_gb'),
        updated_at=pulumi.get(__ret__, 'updated_at'))


@_utilities.lift_output_func(get_disk)
def get_disk_output(disk_id: Optional[pulumi.Input[str]] = None,
                    opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[DiskWithCursorpropertiesdisk]:
    """
    Use this data source to access information about an existing resource.

    :param str disk_id: The ID of the disk
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from enum import Enum

__all__ = [
    'OwnerType',
]


class OwnerType(str, Enum):
    USER = "user"
    TEAM = "team"



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload, Awaitable
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'RedisDetail',
    'AwaitableRedisDetail',
    'get_redis',
    'get_redis_output',
]

@pulumi.output_type
class RedisDetail:
    """
    A Redis instance
    """
    def __init__(__self__, created_at=None, environment_id=None, id=None, ip_allow_list=None, maintenance=None, name=None, options=None, owner=None, plan=None, region=None, status=None, updated_at=None, version=None):
        if created_at and not isinstance(created_at, str):
            raise TypeError("Expected argument 'created_at' to be a str")
        pulumi.set(__self__, "created_at", created_at)
        if environment_id and not isinstance(environment_id, str):
            raise TypeError("Expected argument 'environment_id' to be a str")
        pulumi.set(__self__, "environment_id", environment_id)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if ip_allow_list and not isinstance(ip_allow_list, list):
            raise TypeError("Expected argument 'ip_allow_list' to be a list")
        pulumi.set(__self__, "ip_allow_list", ip_allow_list)
        if maintenance and not isinstance(maintenance, dict):
            raise TypeError("Expected argument 'maintenance' to be a dict")
        pulumi.set(__self__, "maintenance", maintenance)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if options and not isinstance(options, dict):
            raise TypeError("Expected argument 'options' to be a dict")
        pulumi.set(__self__, "options", options)
        if owner and not isinstance(owner, dict):
            raise TypeError("Expected argument 'owner' to be a dict")
        pulumi.set(__self__, "owner", owner)
        if plan and not isinstance(plan, str):
            raise TypeError("Expected argument 'plan' to be a str")
        pulumi.set(__self__, "plan", plan)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)
        if status and not isinstance(status, str):
            raise TypeError("Expected argument 'status' to be a str")
        pulumi.set(__self__, "status", status)
        if updated_at and not isinstance(updated_at, str):
            raise TypeError("Expected argument 'updated_at' to be a str")
        pulumi.set(__self__, "updated_at", updated_at)
        if version and not isinstance(version, str):
            raise TypeError("Expected argument 'version' to be a str")
        pulumi.set(__self__, "version", version)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> str:
        """
        The creation time of the Redis instance
        """
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter(name="environmentId")
    def environment_id(self) -> Optional[str]:
        """
        The ID of the environment the Redis instance is associated with
        """
        return pulumi.get(self, "environment_id")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The ID of the Redis instance
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="ipAllowList")
    def ip_allow_list(self) -> Sequence['outputs.CidrBlockAndDescription']:
        """
        The IP allow list for the Redis instance
        """
        return pulumi.get(self, "ip_allow_list")

    @property
    @pulumi.getter
    def maintenance(self) -> Optional['outputs.RedisDetailMaintenanceProperties']:
        return pulumi.get(self, "maintenance")

    @property
    @pulumi.getter
    def name(self) -> str:
        """
        The name of the Redis instance
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def options(self) -> 'outputs.RedisOptions':
        """
        Options for a Redis instance
        """
        return pulumi.get(self, "options")

    @property
    @pulumi.getter
    def owner(self) -> 'outputs.Owner':
        return pulumi.get(self, "owner")

    @property
    @pulumi.getter
    def plan(self) -> 'RedisDetailPlan':
        return pulumi.get(self, "plan")

    @property
    @pulumi.getter
    def region(self) -> 'RedisDetailRegion':
        """
        Defaults to "oregon"
        """
        return pulumi.get(self, "region")

    @property
    @pulumi.getter
    def status(self) -> 'RedisDetailStatus':
        return pulumi.get(self, "status")

    @property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> str:
        """
        The last updated time of the Redis instance
        """
        return pulumi.get(self, "updated_at")

    @property
    @pulumi.getter
    def version(self) -> str:
        """
        The version of Redis
        """
        return pulumi.get(self, "version")


class AwaitableRedisDetail(RedisDetail):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return RedisDetail(
            created_at=self.created_at,
            environment_id=self.environment_id,
            id=self.id,
            ip_allow_list=self.ip_allow_list,
            maintenance=self.maintenance,
            name=self.name,
            options=self.options,
            owner=self.owner,
            plan=self.plan,
            region=self.region,
            status=self.status,
            updated_at=self.updated_at,
            version=self.version)


def get_redis(redis_id: Optional[str] = None,
              opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableRedisDetail:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['redisId'] = redis_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:redis:getRedis', __args__, opts=opts, typ=RedisDetail).value

    return AwaitableRedisDetail(
        created_at=pulumi.get(__ret__, 'created_at'),
        environment_id=pulumi.get(__ret__, 'environment_id'),
        id=pulumi.get(__ret__, 'id'),
        ip_allow_list=pulumi.get(__ret__, 'ip_allow_list'),
        maintenance=pulumi.get(__ret__, 'maintenance'),
        name=pulumi.get(__ret__, 'name'),
        options=pulumi.get(__ret__, 'options'),
        owner=pulumi.get(__ret__, 'owner'),
        plan=pulumi.get(__ret__, 'plan'),
        region=pulumi.get(__ret__, 'region'),
        status=pulumi.get(__ret__, 'status'),
        updated_at=pulumi.get(__ret__, 'updated_at'),
        version=pulumi.get(__ret__, 'version'))


@_utilities.lift_output_func(get_redis)
def get_redis_output(redis_id: Optional[pulumi.Input[str]] = None,
                     opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[RedisDetail]:
    """
    Use this data source to access information about an existing resource.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs

__all__ = [
    'DiskWithCursor',
    'DiskWithCursorDiskProperties',
]

@pulumi.output_type
class DiskWithCursor(dict):
    def __init__(__self__, *,
                 cursor: str,
                 disk: 'outputs.DiskWithCursorDiskProperties'):
        pulumi.set(__self__, "cursor", cursor)
        pulumi.set(__self__, "disk", disk)

    @property
    @pulumi.getter
    def cursor(self) -> str:
        return pulumi.get(self, "cursor")

    @property
    @pulumi.getter
    def disk(self) -> 'outputs.DiskWithCursorDiskProperties':
        return pulumi.get(self, "disk")


@pulumi.output_type
class DiskWithCursorDiskProperties(dict):
    def __init__(__self__, *,
                 created_at: str,
                 id: str,
                 mount_path: str,
                 name: str,
                 size_gb: int,
                 updated_at: str,
                 service_id: Optional[str] = None):
        pulumi.set(__self__, "created_at", created_at)
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "mount_path", mount_path)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "size_gb", size_gb)
        pulumi.set(__self__, "updated_at", updated_at)
        if service_id is not None:
            pulumi.set(__self__, "service_id", service_id)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> str:
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="mountPath")
    def mount_path(self) -> str:
        return pulumi.get(self, "mount_path")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="sizeGB")
    def size_gb(self) -> int:
        return pulumi.get(self, "size_gb")

    @property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> str:
        return pulumi.get(self, "updated_at")

    @property
    @pulumi.getter(name="serviceId")
    def service_id(self) -> Optional[str]:
        return pulumi.get(self, "service_id")





# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload, Awaitable
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'PostgresDetail',
    'AwaitablePostgresDetail',
    'get_postgres',
    'get_postgres_output',
]

@pulumi.output_type
class PostgresDetail:
    def __init__(__self__, created_at=None, dashboard_url=None, database_name=None, database_user=None, disk_size_gb=None, environment_id=None, expires_at=None, high_availability_enabled=None, id=None, ip_allow_list=None, maintenance=None, name=None, owner=None, plan=None, primary_postgres_id=None, read_replicas=None, region=None, role=None, status=None, suspended=None, suspenders=None, updated_at=None, version=None):
        if created_at and not isinstance(created_at, str):
            raise TypeError("Expected argument 'created_at' to be a str")
        pulumi.set(__self__, "created_at", created_at)
        if dashboard_url and not isinstance(dashboard_url, str):
            raise TypeError("Expected argument 'dashboard_url' to be a str")
        pulumi.set(__self__, "dashboard_url", dashboard_url)
        if database_name and not isinstance(database_name, str):
            raise TypeError("Expected argument 'database_name' to be a str")
        pulumi.set(__self__, "database_name", database_name)
        if database_user and not isinstance(database_user, str):
            raise TypeError("Expected argument 'database_user' to be a str")
        pulumi.set(__self__, "database_user", database_user)
        if disk_size_gb and not isinstance(disk_size_gb, int):
            raise TypeError("Expected argument 'disk_size_gb' to be a int")
        pulumi.set(__self__, "disk_size_gb", disk_size_gb)
        if environment_id and not isinstance(environment_id, str):
            raise TypeError("Expected argument 'environment_id' to be a str")
        pulumi.set(__self__, "environment_id", environment_id)
        if expires_at and not isinstance(expires_at, str):
            raise TypeError("Expected argument 'expires_at' to be a str")
        pulumi.set(__self__, "expires_at", expires_at)
        if high_availability_enabled and not isinstance(high_availability_enabled, bool):
            raise TypeError("Expected argument 'high_availability_enabled' to be a bool")
        pulumi.set(__self__, "high_availability_enabled", high_availability_enabled)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if ip_allow_list and not isinstance(ip_allow_list, list):
            raise TypeError("Expected argument 'ip_allow_list' to be a list")
        pulumi.set(__self__, "ip_allow_list", ip_allow_list)
        if maintenance and not isinstance(maintenance, dict):
            raise TypeError("Expected argument 'maintenance' to be a dict")
        pulumi.set(__self__, "maintenance", maintenance)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if owner and not isinstance(owner, dict):
            raise TypeError("Expected argument 'owner' to be a dict")
        pulumi.set(__self__, "owner", owner)
        if plan and not isinstance(plan, str):
            raise TypeError("Expected argument 'plan' to be a str")
        pulumi.set(__self__, "plan", plan)
        if primary_postgres_id and not isinstance(primary_postgres_id, str):
            raise TypeError("Expected argument 'primary_postgres_id' to be a str")
        pulumi.set(__self__, "primary_postgres_id", primary_postgres_id)
        if read_replicas and not isinstance(read_replicas, list):
            raise TypeError("Expected argument 'read_replicas' to be a list")
        pulumi.set(__self__, "read_replicas", read_replicas)
        if region and not isinstance(region, str):
            raise TypeError("Expected argument 'region' to be a str")
        pulumi.set(__self__, "region", region)
        if role and not isinstance(role, str):
            raise TypeError("Expected argument 'role' to be a str")
        pulumi.set(__self__, "role", role)
        if status and not isinstance(status, str):
            raise TypeError("Expected argument 'status' to be a str")
        pulumi.set(__self__, "status", status)
        if suspended and not isinstance(suspended, str):
            raise TypeError("Expected argument 'suspended' to be a str")
        pulumi.set(__self__, "suspended", suspended)
        if suspenders and not isinstance(suspenders, list):
            raise TypeError("Expected argument 'suspenders' to be a list")
        pulumi.set(__self__, "suspenders", suspenders)
        if updated_at and not isinstance(updated_at, str):
            raise TypeError("Expected argument 'updated_at' to be a str")
        pulumi.set(__self__, "updated_at", updated_at)
        if version and not isinstance(version, str):
            raise TypeError("Expected argument 'version' to be a str")
        pulumi.set(__self__, "version", version)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> str:
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter(name="dashboardUrl")
    def dashboard_url(self) -> str:
        """
        The URL to view the PostgreSQL instance in the Render Dashboard
        """
        return pulumi.get(self, "dashboard_url")

    @property
    @pulumi.getter(name="databaseName")
    def database_name(self) -> str:
        return pulumi.get(self, "database_name")

    @property
    @pulumi.getter(name="databaseUser")
    def database_user(self) -> str:
        return pulumi.get(self, "database_user")

    @property
    @pulumi.getter(name="diskSizeGB")
    def disk_size_gb(self) -> Optional[int]:
        return pulumi.get(self, "disk_size_gb")

    @property
    @pulumi.getter(name="environmentId")
    def environment_id(self) -> Optional[str]:
        return pulumi.get(self, "environment_id")

    @property
    @pulumi.getter(name="expiresAt")
    def expires_at(self) -> Optional[str]:
        """
        The time at which the database will be expire. Applies to free tier databases only.
        """
        return pulumi.get(self, "expires_at")

    @property
    @pulumi.getter(name="highAvailabilityEnabled")
    def high_availability_enabled(self) -> bool:
        return pulumi.get(self, "high_availability_enabled")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="ipAllowList")
    def ip_allow_list(self) -> Sequence['outputs.CidrBlockAndDescription']:
        return pulumi.get(self, "ip_allow_list")

    @property
    @pulumi.getter
    def maintenance(self) -> Optional['outputs.RedisDetailpropertiesmaintenance']:
        return pulumi.get(self, "maintenance")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def owner(self) -> 'outputs.Owner':
        return pulumi.get(self, "owner")

    @property
    @pulumi.getter
    def plan(self) -> 'PostgresDetailPlan':
        return pulumi.get(self, "plan")

    @property
    @pulumi.getter(name="primaryPostgresID")
    def primary_postgres_id(self) -> Optional[str]:
        return pulumi.get(self, "primary_postgres_id")

    @property
    @pulumi.getter(name="readReplicas")
    def read_replicas(self) -> Sequence['outputs.ReadReplica']:
        return pulumi.get(self, "read_replicas")

    @property
    @pulumi.getter
    def region(self) -> 'PostgresDetailRegion':
        """
        Defaults to "oregon"
        """
        return pulumi.get(self, "region")

    @property
    @pulumi.getter
    def role(self) -> 'PostgresDetailRole':
        return pulumi.get(self, "role")

    @property
    @pulumi.getter
    def status(self) -> 'PostgresDetailStatus':
        return pulumi.get(self, "status")

    @property
    @pulumi.getter
    def suspended(self) -> 'PostgresDetailSuspended':
        return pulumi.get(self, "suspended")

    @property
    @pulumi.getter
    def suspenders(self) -> Sequence['PostgresDetailSuspendersItem']:
        return pulumi.get(self, "suspenders")

    @property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> str:
        return pulumi.get(self, "updated_at")

    @property
    @pulumi.getter
    def version(self) -> 'PostgresDetailVersion':
        """
        The PostgreSQL version
        """
        return pulumi.get(self, "version")


class AwaitablePostgresDetail(PostgresDetail):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return PostgresDetail(
            created_at=self.created_at,
            dashboard_url=self.dashboard_url,
            database_name=self.database_name,
            database_user=self.database_user,
            disk_size_gb=self.disk_size_gb,
            environment_id=self.environment_id,
            expires_at=self.expires_at,
            high_availability_enabled=self.high_availability_enabled,
            id=self.id,
            ip_allow_list=self.ip_allow_list,
            maintenance=self.maintenance,
            name=self.name,
            owner=self.owner,
            plan=self.plan,
            primary_postgres_id=self.primary_postgres_id,
            read_replicas=self.read_replicas,
            region=self.region,
            role=self.role,
            status=self.status,
            suspended=self.suspended,
            suspenders=self.suspenders,
            updated_at=self.updated_at,
            version=self.version)


def get_postgres(postgres_id: Optional[str] = None,
                 opts: Optional[pulumi.InvokeOptions] = None) -> AwaitablePostgresDetail:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['postgresId'] = postgres_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:postgres:getPostgres', __args__, opts=opts, typ=PostgresDetail).value

    return AwaitablePostgresDetail(
        created_at=pulumi.get(__ret__, 'created_at'),
        dashboard_url=pulumi.get(__ret__, 'dashboard_url'),
        database_name=pulumi.get(__ret__, 'database_name'),
        database_user=pulumi.get(__ret__, 'database_user'),
        disk_size_gb=pulumi.get(__ret__, 'disk_size_gb'),
        environment_id=pulumi.get(__ret__, 'environment_id'),
        expires_at=pulumi.get(__ret__, 'expires_at'),
        high_availability_enabled=pulumi.get(__ret__, 'high_availability_enabled'),
        id=pulumi.get(__ret__, 'id'),
        ip_allow_list=pulumi.get(__ret__, 'ip_allow_list'),
        maintenance=pulumi.get(__ret__, 'maintenance'),
        name=pulumi.get(__ret__, 'name'),
        owner=pulumi.get(__ret__, 'owner'),
        plan=pulumi.get(__ret__, 'plan'),
        primary_postgres_id=pulumi.get(__ret__, 'primary_postgres_id'),
        read_replicas=pulumi.get(__ret__, 'read_replicas'),
        region=pulumi.get(__ret__, 'region'),
        role=pulumi.get(__ret__, 'role'),
        status=pulumi.get(__ret__, 'status'),
        suspended=pulumi.get(__ret__, 'suspended'),
        suspenders=pulumi.get(__ret__, 'suspenders'),
        updated_at=pulumi.get(__ret__, 'updated_at'),
        version=pulumi.get(__ret__, 'version'))


@_utilities.lift_output_func(get_postgres)
def get_postgres_output(postgres_id: Optional[pulumi.Input[str]] = None,
                        opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[PostgresDetail]:
    """
    Use this data source to access information about an existing resource.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'BlueprintWithCursor',
    'BlueprintWithCursorBlueprintProperties',
    'GetBlueprintPropertiesResourcesItemProperties',
    'SyncWithCursor',
    'SyncWithCursorSyncProperties',
    'SyncWithCursorSyncPropertiesCommitProperties',
]

@pulumi.output_type
class BlueprintWithCursor(dict):
    """
    A blueprint with a cursor
    """
    def __init__(__self__, *,
                 blueprint: 'outputs.BlueprintWithCursorBlueprintProperties',
                 cursor: str):
        """
        A blueprint with a cursor
        """
        pulumi.set(__self__, "blueprint", blueprint)
        pulumi.set(__self__, "cursor", cursor)

    @property
    @pulumi.getter
    def blueprint(self) -> 'outputs.BlueprintWithCursorBlueprintProperties':
        return pulumi.get(self, "blueprint")

    @property
    @pulumi.getter
    def cursor(self) -> str:
        return pulumi.get(self, "cursor")


@pulumi.output_type
class BlueprintWithCursorBlueprintProperties(dict):
    def __init__(__self__, *,
                 auto_sync: bool,
                 branch: str,
                 id: str,
                 name: str,
                 repo: str,
                 status: 'BlueprintWithCursorBlueprintPropertiesStatus',
                 last_sync: Optional[str] = None):
        """
        :param bool auto_sync: Automatically sync changes to render.yaml
        """
        pulumi.set(__self__, "auto_sync", auto_sync)
        pulumi.set(__self__, "branch", branch)
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "repo", repo)
        pulumi.set(__self__, "status", status)
        if last_sync is not None:
            pulumi.set(__self__, "last_sync", last_sync)

    @property
    @pulumi.getter(name="autoSync")
    def auto_sync(self) -> bool:
        """
        Automatically sync changes to render.yaml
        """
        return pulumi.get(self, "auto_sync")

    @property
    @pulumi.getter
    def branch(self) -> str:
        return pulumi.get(self, "branch")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def repo(self) -> str:
        return pulumi.get(self, "repo")

    @property
    @pulumi.getter
    def status(self) -> 'BlueprintWithCursorBlueprintPropertiesStatus':
        return pulumi.get(self, "status")

    @property
    @pulumi.getter(name="lastSync")
    def last_sync(self) -> Optional[str]:
        return pulumi.get(self, "last_sync")


@pulumi.output_type
class GetBlueprintPropertiesResourcesItemProperties(dict):
    def __init__(__self__, *,
                 id: str,
                 name: str,
                 type: 'GetBlueprintPropertiesResourcesItemPropertiesType'):
        """
        :param 'GetBlueprintPropertiesResourcesItemPropertiesType' type: type of the resource (ex. web_service or postgres)
        """
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "type", type)

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def type(self) -> 'GetBlueprintPropertiesResourcesItemPropertiesType':
        """
        type of the resource (ex. web_service or postgres)
        """
        return pulumi.get(self, "type")


@pulumi.output_type
class SyncWithCursor(dict):
    """
    A blueprint sync with a cursor
    """
    def __init__(__self__, *,
                 cursor: str,
                 sync: 'outputs.SyncWithCursorSyncProperties'):
        """
        A blueprint sync with a cursor
        """
        pulumi.set(__self__, "cursor", cursor)
        pulumi.set(__self__, "sync", sync)

    @property
    @pulumi.getter
    def cursor(self) -> str:
        return pulumi.get(self, "cursor")

    @property
    @pulumi.getter
    def sync(self) -> 'outputs.SyncWithCursorSyncProperties':
        return pulumi.get(self, "sync")


@pulumi.output_type
class SyncWithCursorSyncProperties(dict):
    def __init__(__self__, *,
                 commit: 'outputs.SyncWithCursorSyncPropertiesCommitProperties',
                 id: str,
                 state: 'SyncWithCursorSyncPropertiesState',
                 completed_at: Optional[str] = None,
                 started_at: Optional[str] = None):
        pulumi.set(__self__, "commit", commit)
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "state", state)
        if completed_at is not None:
            pulumi.set(__self__, "completed_at", completed_at)
        if started_at is not None:
            pulumi.set(__self__, "started_at", started_at)

    @property
    @pulumi.getter
    def commit(self) -> 'outputs.SyncWithCursorSyncPropertiesCommitProperties':
        return pulumi.get(self, "commit")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def state(self) -> 'SyncWithCursorSyncPropertiesState':
        return pulumi.get(self, "state")

    @property
    @pulumi.getter(name="completedAt")
    def completed_at(self) -> Optional[str]:
        return pulumi.get(self, "completed_at")

    @property
    @pulumi.getter(name="startedAt")
    def started_at(self) -> Optional[str]:
        return pulumi.get(self, "started_at")


@pulumi.output_type
class SyncWithCursorSyncPropertiesCommitProperties(dict):
    def __init__(__self__, *,
                 id: str):
        pulumi.set(__self__, "id", id)

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")





# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from .. import _utilities
import typing
# Export this package's modules as members:
from ._enums import *
from .failover_postgres import *
from .get_postgres import *
from .get_postgres_connection_info import *
from .get_postgres_recovery_info import *
from .list_postgres import *
from .list_postgres_backup import *
from .postgres import *
from .postgres_backup import *
from .recover_postgres import *
from .restart_postgres import *
from .resume_postgres import *
from .suspend_postgres import *
from ._inputs import *
from . import outputs



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload, Awaitable
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'Project',
    'AwaitableProject',
    'get_project',
    'get_project_output',
]

@pulumi.output_type
class Project:
    """
    A project is a collection of environments
    """
    def __init__(__self__, created_at=None, environment_ids=None, id=None, name=None, owner=None, updated_at=None):
        if created_at and not isinstance(created_at, str):
            raise TypeError("Expected argument 'created_at' to be a str")
        pulumi.set(__self__, "created_at", created_at)
        if environment_ids and not isinstance(environment_ids, list):
            raise TypeError("Expected argument 'environment_ids' to be a list")
        pulumi.set(__self__, "environment_ids", environment_ids)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if name and not isinstance(name, str):
            raise TypeError("Expected argument 'name' to be a str")
        pulumi.set(__self__, "name", name)
        if owner and not isinstance(owner, dict):
            raise TypeError("Expected argument 'owner' to be a dict")
        pulumi.set(__self__, "owner", owner)
        if updated_at and not isinstance(updated_at, str):
            raise TypeError("Expected argument 'updated_at' to be a str")
        pulumi.set(__self__, "updated_at", updated_at)

    @property
    @pulumi.getter(name="createdAt")
    def created_at(self) -> str:
        return pulumi.get(self, "created_at")

    @property
    @pulumi.getter(name="environmentIds")
    def environment_ids(self) -> Sequence[str]:
        """
        The environments associated with the project
        """
        return pulumi.get(self, "environment_ids")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The ID of the project
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        """
        The name of the project
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def owner(self) -> 'outputs.Owner':
        return pulumi.get(self, "owner")

    @property
    @pulumi.getter(name="updatedAt")
    def updated_at(self) -> str:
        return pulumi.get(self, "updated_at")


class AwaitableProject(Project):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return Project(
            created_at=self.created_at,
            environment_ids=self.environment_ids,
            id=self.id,
            name=self.name,
            owner=self.owner,
            updated_at=self.updated_at)


def get_project(project_id: Optional[str] = None,
                opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableProject:
    """
    Use this data source to access information about an existing resource.
    """
    __args__ = dict()
    __args__['projectId'] = project_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('render:projects:getProject', __args__, opts=opts, typ=Project).value

    return AwaitableProject(
        created_at=pulumi.get(__ret__, 'created_at'),
        environment_ids=pulumi.get(__ret__, 'environment_ids'),
        id=pulumi.get(__ret__, 'id'),
        name=pulumi.get(__ret__, 'name'),
        owner=pulumi.get(__ret__, 'owner'),
        updated_at=pulumi.get(__ret__, 'updated_at'))


@_utilities.lift_output_func(get_project)
def get_project_output(project_id: Optional[pulumi.Input[str]] = None,
                       opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[Project]:
    """
    Use this data source to access information about an existing resource.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._enums import *

__all__ = [
    'ListOwnersItemProperties',
    'Owner',
]

@pulumi.output_type
class ListOwnersItemProperties(dict):
    def __init__(__self__, *,
                 cursor: Optional[str] = None,
                 owner: Optional['outputs.Owner'] = None):
        if cursor is not None:
            pulumi.set(__self__, "cursor", cursor)
        if owner is not None:
            pulumi.set(__self__, "owner", owner)

    @property
    @pulumi.getter
    def cursor(self) -> Optional[str]:
        return pulumi.get(self, "cursor")

    @property
    @pulumi.getter
    def owner(self) -> Optional['outputs.Owner']:
        return pulumi.get(self, "owner")


@pulumi.output_type
class Owner(dict):
    def __init__(__self__, *,
                 email: str,
                 id: str,
                 name: str,
                 type: 'OwnerType',
                 two_factor_auth_enabled: Optional[bool] = None):
        """
        :param bool two_factor_auth_enabled: Whether two-factor authentication is enabled for the owner. Only present for user owners.
        """
        pulumi.set(__self__, "email", email)
        pulumi.set(__self__, "id", id)
        pulumi.set(__self__, "name", name)
        pulumi.set(__self__, "type", type)
        if two_factor_auth_enabled is not None:
            pulumi.set(__self__, "two_factor_auth_enabled", two_factor_auth_enabled)

    @property
    @pulumi.getter
    def email(self) -> str:
        return pulumi.get(self, "email")

    @property
    @pulumi.getter
    def id(self) -> str:
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def type(self) -> 'OwnerType':
        return pulumi.get(self, "type")

    @property
    @pulumi.getter(name="twoFactorAuthEnabled")
    def two_factor_auth_enabled(self) -> Optional[bool]:
        """
        Whether two-factor authentication is enabled for the owner. Only present for user owners.
        """
        return pulumi.get(self, "two_factor_auth_enabled")





# coding=utf-8
# *** WARNING: this file was generated by pulumigen. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = ['ScaleServiceArgs', 'ScaleService']

@pulumi.input_type
class ScaleServiceArgs:
    def __init__(__self__, *,
                 num_instances: pulumi.Input[int],
                 service_id: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a ScaleService resource.
        :param pulumi.Input[str] service_id: The ID of the service
        """
        pulumi.set(__self__, "num_instances", num_instances)
        if service_id is not None:
            pulumi.set(__self__, "service_id", service_id)

    @property
    @pulumi.getter(name="numInstances")
    def num_instances(self) -> pulumi.Input[int]:
        return pulumi.get(self, "num_instances")

    @num_instances.setter
    def num_instances(self, value: pulumi.Input[int]):
        pulumi.set(self, "num_instances", value)

    @property
    @pulumi.getter(name="serviceId")
    def service_id(self) -> Optional[pulumi.Input[str]]:
        """
        The ID of the service
        """
        return pulumi.get(self, "service_id")

    @service_id.setter
    def service_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "service_id", value)


class ScaleService(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 num_instances: Optional[pulumi.Input[int]] = None,
                 service_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Create a ScaleService resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] service_id: The ID of the service
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ScaleServiceArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Create a ScaleService resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param ScaleServiceArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ScaleServiceArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 num_instances: Optional[pulumi.Input[int]] = None,
                 service_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ScaleServiceArgs.__new__(ScaleServiceArgs)

            if num_instances is None and not opts.urn:
                raise TypeError("Missing required property 'num_instances'")
            __props__.__dict__["num_instances"] = num_instances
            __props__.__dict__["service_id"] = service_id
        super(ScaleService, __self__).__init__(
            'render:services:ScaleService',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None) -> 'ScaleService':
        """
        Get an existing ScaleService resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = ScaleServiceArgs.__new__(ScaleServiceArgs)

        __props__.__dict__["num_instances"] = None
        return ScaleService(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="numInstances")
    def num_instances(self) -> pulumi.Output[int]:
        return pulumi.get(self, "num_instances")

